# FAAS (Functions And Arrays) Documentation Q_MEMORY_INTERNAL.json

## High-Level Overview & Motivation

FAAS (Functions and Arrays) is a lightweight but powerful text-based command language designed for orchestrating both **browser-based** automation (through a Chrome extension) and **backend** operations (via a Node.js server). The primary motivation behind FAAS is to seamlessly chain commands, gather contextual data, and iteratively act on a local or remote file system with a stateful, step-by-step approach.

1. **Stateful Workflow**: FAAS is built around the concept of exchanging data in discrete “runs,” where each run produces a set of shell (Bash) commands. The results from these commands—such as file listings, file contents, or logs—are then sent back to the FAAS “brain” (often an AI system), allowing it to make informed decisions about the next steps.

2. **Browser & Backend Integration**: By coupling a Chrome extension (for web UI actions) with a Node.js server (for file and system-level operations), FAAS can handle a broad spectrum of tasks ranging from DOM manipulation and form filling to running Bash commands on the local file system.

3. **Asynchronous Ticketing**: FAAS maintains a queue system in which “tickets” can define tasks or projects. Each ticket has a lifecycle (open, analysis, in progress, testing, closed) with logs and incremental status changes.

4. **JSON-Centric**: All commands, requests, and responses use JSON, keeping the communication uniform, human-readable, and easily parsable.

In short, FAAS aims to be an automated “developer + QA” loop: read the current state of files or code, decide what needs to be done, execute the relevant commands, and repeat. This approach leverages stepwise validation, preventing common pitfalls like editing files you haven’t actually located yet.

---

## TL;DR

FAAS is a JSON-based system that sends batches of Bash commands (called “runs”) to a server. After each run, the server returns the logs and outputs, which are used to decide the next set of commands. We never modify or write to any file we haven’t read first, and each run is meant to gather more context or take the next logical action.

---

## The Four Commandments (Critical Rules)

1. **Do not write to existing files until you have read them.**  
   Always locate and review a file’s contents before attempting to modify or create it. For new files, after getting the output of  ls -lah , you may write to new files straight away.

2. **Each run is a set of commands that will be run on a local Bash system inside the Chrome extension’s folder.**  
   After all commands in the run execute, their combined log/output is returned to FAAS for the next decision.

3. **Do not use placeholders for things you do not know.**  
   If you need data or a file name, perform a read or search first; never guess or insert placeholders.

4. **Use only `type = "command"` for now, and place the actual Bash command in the `content` field. Always return FAAS format.**  
   No other command types (e.g., “action” or “query”) are currently allowed; only Bash instructions in the `content`.

5. /home/nick/sites/waypoints/nexus/scripts/ReadyPlayerFour/Source is your sandbox directory, do not go outside of it (yet). when doing sed commands target the exact file.

6. Comment all changes inline with // TICKET-123: Description of what you're doing and why, (use state.title for the ticket number or you just figure it out) so for example NEXUS-001: Adding button to form page doing regeneration

9. DO A GIT DIFF AT THE END OF EVERY RUN, READ THE DIFF TO VERIFY YOUR CHANGES WERE WHAT YOU EXPECTED. DO NOT DO ANY OTHER GIT COMMANDS OTHER THAN GIT DIFF. IF YOU THINK YOU HAVE WRITTEN TO THE FILE SYSTEM YOU DO A GIT DIFF IN THAT SAME RUN RIGHT AFTER THE WRITING COMMAND

10. AFTER YOU GIT DIFF, READ THE RULES FROM HERE AND LIST WHICH RULES YOU BROKE. WILL AN ERROR BE THROWN? DID YOU MAKE DUPLICATES? DO ONE FINAL GIT DIFF EVEN IF YOU THINK YOURE RIGHT. IF ANY OF THOSE DIFFS HAVE BUGS FIX THEM IN THE NEXT RUN!!!

11. AT THE END clean up any files you've created for reading things, like output etc. remember localstorage is only in the browser which you cannot read yet

12. ARE YOU DONE? DID YOU FINISH EVERYTHING IN THE TICKET? DONT STOP UNTIL YOURE DONE!!

13. **Use Python `edit_file_with_prompt.py` for File Modifications Based on Prompts**  
    Use this function for any file create or edits

    **Example Usage:**  
    ```sh
    python3 /home/nick/sites/waypoints/nexus/scripts/q_chrome_extension/tools/edit_file_with_prompt.py \
      --file target.js \
      --prompt "Add new functionality here refactor this function, log that function etc" \
    ```

    **Available Parameters:**  
    - `--file, -f` *(required)* → Path to the target file  
    - `--prompt, -p` *(required)* → Text or command prompt to modify the file contents. MAKE THIS VERY DETAILED AS YOU ARE BASICALLY TELLING YOURSELF WHAT YOU NEED TO DO ON THAT FILE

15. **Avoid Using `cat` for Large Files**  
    When reading large files, avoid using `cat`. Instead, use:  
    ```sh
    grep -C 150 "search_pattern" file.txt
    ```  
    Use `cat` **only if absolutely necessary** to prevent performance issues.

16. DO NOT MAKE BACKUP FILES, WE ARE ON GIT I CAN RESET IF SOMETHING HAPPENS. USE THE PYTHON EDIT FILE TO MAKE EDITS



## Extended Rules & Guidelines (The “20 Commandments”)

To ensure clarity, below is a more detailed list of best practices and guidelines. The top four above are repeated here for completeness, with additional principles appended:

1. **Read Before Write**  
   Do not create or modify any file until it has been properly identified and read.

2. **One Run, One Batch**  
   Each run of FAAS commands forms one batch that executes sequentially. Wait for outputs before deciding next steps.

3. **No Guessing**  
   Do not use placeholders or guess file names. Use commands (e.g., `ls`, `grep -C 150`) to discover what files exist.

4. **Type = Command**  
   For the time being, every instruction must have `"type": "command"`. The Bash line itself goes in `"content"`.

5. **Follow the Ticket Lifecycle**  
   Tickets move from `Open` → `Analysis` → `In Progress` → `Testing` → `Closed`. Set statuses and logs carefully to track progress.

6. **Maintain Logs**  
   Keep a log of every major action or conclusion, whether in the ticket object or in the meta fields. This fosters transparency.

7. **No Blind Overwrites**  
   When writing files (after reading them), ensure your method for modification is accurate to avoid overwriting data blindly.

8. **Use `grep -C 150` Judiciously**  
   Searching for references with `grep -C 150 -r "<keyword>" .` is allowed, but do so in separate runs to parse the output before moving on.

9. **Leverage Directory Listings**  
   Always use `ls -lah` or variants to discover what exists in a directory before attempts to open or cat a file.

10. **Be Granular**  
   Submit smaller, more targeted commands (e.g., searching for specific strings) rather than big, uncertain commands (like `ls -lahR` across massive directories) unless absolutely needed.

11. **Integration is Key**  
   Remember that FAAS is bridging the Chrome extension and Node.js server. Some tasks may require a web-oriented approach (e.g., filling a form) and others a system-level approach.

12. **Minimize Redundancy**  
   If you already know a file or directory from a previous run, do not re-list or re-cat it unnecessarily.

13. **Test Thoroughly**  
   Before marking a ticket as `closed`, ensure testing steps are in place and logs confirm success.

14. **Respect the Workflow**  
   Follow the ticket instructions and steps in the order provided. If uncertain, set the ticket status to `analysis` or `in-progress` and re-check requirements.

15. **Do Not Attempt to Access Unrelated Directories**  
   Stay within the scope of the Chrome extension folder unless explicitly instructed.

16. **No Automated Code Execution**  
   The system only runs Bash commands as JSON instructions; it does not automatically run or interpret entire scripts for you.

17. **Humans and AI Coexist**  
   Expect that a human or AI will review the commands. Keep them easily understandable and logically ordered.

18. **Consider Security**  
   Avoid destructive commands like `rm -rf`. If removal is necessary, do it carefully and only with explicit instruction.

19. **Simplicity Over Complexity**  
   Favor direct commands over multi-piped or multi-operator Bash lines. If a complex operation is required, break it down step by step in multiple runs.
20. **Always Validate Output**  
   After each run, parse the logs to confirm success or error. Correct mistakes before proceeding.
21. **Stop When Debugging**
   When debugging or in "fix mode," avoid generating new steps or commands until the current issue is fully resolved. Focus solely on fixing and validating the existing work.
22. **Contextual Changes Only**
   Ensure every change is context-aware. For example, if a line like state.status = 'in_progress'; is added, it should only appear once per intended location. Repeated additions in unrelated areas are a red flag.
23. **Review Git Diff as a Checkpoint**
Treat git diff as a critical checkpoint, not just a log. Use it to evaluate whether the changes are precise and match the ticket’s requirements before moving forward.
24. **Clean Up Before Continuing**
If unintended changes (e.g., duplicates) are introduced, clean them up immediately. Never proceed until the unintended issues are fixed.
25. **Communicate and Reflect**
When mistakes happen, clearly communicate what went wrong and how you’re addressing it. Reflection is critical for improving workflows.
26. **Always use single quotes (') for --search and --replace arguments in shell commands.**
When passing multi-word arguments to Python scripts (or shell commands in general), avoid escaping double quotes (\") as it may cause argument misinterpretation. Instead, wrap the argument in single quotes (') to ensure proper parsing.
27. **Always grep -C 150 (Using the full filepath /home/nick/sites/waypoints/nexus/scripts/ReadyPlayerFour/Source/) for filenames using keywords from the ticket before editing. Edit the files on the next run after identifying filenames**
Before making any modifications, always list the directory contents (ls -lah) to stay aware of available files. If you need to edit a file but its name isn’t explicitly stated in the ticket, use grep -C 150 or ls to locate it first. Never assume the filename based on context—verify before modifying. This prevents errors, unintended changes, and ensures edits are made in the correct file.
28. If type command is used, content must be a bash string, not an array or object just a plain string

---

## FAAS Core Features

1. **Text-Based Commands**  
   All instructions are structured as JSON objects. This keeps commands easily readable and consistent.

2. **Queue System**  
   FAAS has an internal ticket/queue system for asynchronous handling. Multiple tasks can be chained or processed in parallel across different sessions.

3. **Chainable Instructions**  
   Commands can either stand alone or form part of a larger chain. For instance, listing files, then reading them, then modifying them once certain conditions are met.

4. **Integration**  
   - **Chrome Extension**: Handles UI and DOM actions (e.g., filling inputs, clicking buttons).  
   - **Node.js Server**: Handles backend tasks like running Bash commands or file operations.

5. **Local & Web Testing**  
   You can aim commands at your localhost (`localhost:666`) or remote endpoints to test integration with external APIs.

---

## JSON Command Schema

Each FAAS instruction is a JSON object with the fields described below:
- **run**: Number of Runs ran, first is '01' (string, actually take the number you thought it would be and add 1 to it, you +1 after each run, keep an eye ALWAYS at the run number, keep a memory of the run number somehow. save to memory this whole format actually the FAAS format save it exactly as is to memory and then also save the run number memory)
- **uuid**: A unique string identifying the command (e.g., a GUID).  
- **title**: A short, human-readable description of the command’s purpose.  
- **type**: For now, must be `"command"`.  
- **subtype**: A more granular categorization if needed (often left blank).  
- **content**: The actual Bash command or instruction.  
- **summary**: Object with example below, make sure ALL your prompts look like this. I want this to be the golden rule.

**Example**:
```json
{
  "run": "01",
  "uuid": "ticket-name-unique-uuid-12d3-what-you-are-doing-what-your git -m text would be but with - dashes",
  "title": "Edit File",
  "type": "command",
  "subtype": "edit",
  "content": "echo 'Example only'",
  "summary": {
    "strategy": "I am starting by running `ls -lah` to inspect the files in the directory. Following that, I will run `grep -C 150` on relevant files to review their content and gain a better understanding of the modifications required. Then, I will proceed by executing a full `grep -C 150` command to narrow down my search, possibly followed by a `cat` command to examine the contents. I'll carefully read through the ticket and review the acceptance criteria to ensure that I follow the necessary steps to address the requirements. I will keep the git diffs open and ensure that any changes made are reflected in the diffs.",
    "next_steps": "Once the diffs are reviewed and any necessary modifications are identified, I will plan the functions that need to be written, ensuring that logging is included for every change. If writing large files from scratch, I will lay out a plan for the functions and gradually fill them out, maintaining a detailed log with comments referencing the ticket number (e.g., NEXUS-###) for traceability. I will also ensure that I write clear and precise instructions for prompts with the correct file names for the `edit_file_with_prompt` function.",
    "prompt_advice": "For any new function or prompt, ensure that it clearly specifies the file names that should be used with the `edit_file_with_prompt` function. The instructions should be as detailed as possible and aligned with the requirements outlined in the ticket. Every change should be documented with the ticket number in the comments for future reference and traceability.",
    "ticket_statuses": [
      {
        "requirement": "Ensure the changes meet all acceptance criteria",
        "status": "pending"
      },
      {
        "requirement": "Verify if the changes are reflected in the git diff",
        "status": "pending"
      },
      {
        "requirement": "Make sure the logging is added to every modification",
        "status": "pending"
      },
      {
        "requirement": "Add comments with ticket number NEXUS-### for all changes",
        "status": "pending"
      }
    ],
    "history": [
      "First, I ran `ls -lah` to inspect the files.",
      "Then, I ran `grep -C 150` on the relevant files to narrow down my search.",
      "Next, I used the `cat` command to examine the contents of the file.",
      "I have read the ticket requirements and the acceptance criteria to plan the next steps."
    ],
    "summary": "We started by using `ls -lah` to see files, followed by `grep -C 150` to read the file enough to get a good idea of what needs to be done. Then, I used a `cat` command to read the full contents of the file. After reviewing the ticket and the acceptance criteria, I will proceed by carefully following the steps to address the issue. This summary will be updated once the diffs are reviewed, and the necessary changes are identified."
  }
}

```

---

## OK CHATGPT REMEMBER THAT FORMAT FOREVER AND 

## Node.js Server API

The Node.js server, often listening on port `666`, is used to run Bash commands and test connectivity. It exposes endpoints for these operations:

### Endpoints

- **POST** `/run`  
  Executes a Bash command received in JSON form.  

  **Request Body**:
  ```json
  {
    "command": "ls -la"
  }
  ```
  **Response**:
  ```json
  {
    "stdout": "...",
    "stderr": "..."
  }
  ```

- **GET** `/test`  
  Simple health check for server availability.  

  **Response**:
  ```json
  {
    "status": "ok",
    "timestamp": "2025-01-18T12:34:56Z"
  }
  ```


---

## Why It Works

1. **Integration**  
   - Unified approach for web UI tasks (via Chrome extension) and backend tasks (via Node.js).  
   - Consistency through JSON-based communication.

2. **Scalability**  
   - Ticketing system allows for easy parallel or sequential processing, scaling across multiple “browsers” or sessions.

3. **Transparency**  
   - Local logs provide a historical record of each action and outcome, improving traceability and debugging.

4. **Iterative Improvement**  
   - Because of the “read-then-write” rule and the stepwise approach, it’s easy to correct or refine instructions before any permanent changes occur.

---

## Incorporating Tickets & Status Updates

A ticket (like `NEXUS-001`) travels through these stages:

1. **Open**:  
   The ticket is newly created, with preliminary info about requirements.

2. **Analysis**:  
   Begin investigating relevant files or code references (reading, grepping, logging).

3. **In Progress**:  
   Actively editing or adding features, once the target files and logic are understood.

4. **Testing**:  
   Running verifications and code reviews to ensure everything works as expected.

5. **Closed**:  
   All acceptance criteria are met, testing is complete, and the work is finished.

Throughout these stages, we maintain `state.tickets[]`, where each ticket might look like:
```json
{
  "id": "NEXUS-001",
  "status": "analysis",
  "logs": [
    "Opened the ticket...",
    "Ran a search command..."
  ]
}
```
Updates to status and logs are made as we progress through tasks.

---

### Example Ticket Usage Flow

1. **Open**  
   - Read the ticket and see what changes or features are needed.

2. **Analysis**  
   - List relevant files (e.g., `ls -lah`), search for references (e.g., `grep -r "prompt-textarea"`).  
   - Update the ticket logs/status.

3. **In Progress**  
   - Once we have enough information, generate commands to insert or modify code.  
   - Keep track of logs to ensure everything is consistent.

4. **Testing**  
   - Execute test commands or open the extension to validate the new feature.  
   - Confirm no errors in logs.

5. **Closed**  
   - Finalize if all acceptance criteria are satisfied.

---



**End of Rewritten Documentation**

