// Define the extension version and update description
const currentVersion = "1.5.3"; // Replace with your current extension version
const updateDescription = `Fixed new update on the UI of ChatGPT causing issues on the Auto Full Mode.`;

// Check if the extension has been updated
chrome.runtime.onInstalled.addListener(async function (details) {
  if (details.reason === "install") {
    // This block will be executed only when the extension is installed
    const tabs = await chrome.tabs.query({ url: "https://chatgpt.com/*" });

    if (tabs.length > 0) {
      await chrome.notifications.create({
        type: "basic",
        iconUrl: "./images/icon128.png",
        title: "Reload OpenAI Tabs?",
        message:
          "Would you like to reload your OpenAI tabs to activate the extension?",
        buttons: [{ title: "Reload" }],
        priority: 2,
      });
    }
  } else if (details.reason === "update") {
    // This block will be executed only when the extension is updated
    const tabs = await chrome.tabs.query({ url: "https://chatgpt.com/*" });

    if (tabs.length > 0) {
      await chrome.notifications.create({
        type: "basic",
        iconUrl: "./images/icon128.png",
        title: "Continue Generating was updated!",
        message:
          "Your extension has been updated to version " +
          currentVersion +
          ". \n" +
          updateDescription,
        buttons: [{ title: "Great! Reload OpenAI Tabs!" }],
        priority: 2,
      });
    }
  }
});

chrome.notifications.onButtonClicked.addListener(async function () {
  const tabs = await chrome.tabs.query({ url: "https://chatgpt.com/*" });

  tabs.forEach((tab) => {
    chrome.tabs.reload(tab.id);
  });
});

chrome.tabs.onUpdated.addListener((tabId, changeInfo, tab) => {
  // Check if the tab has finished loading
  if (changeInfo.status === 'complete') {
    console.log('Page loaded, injecting scripts into tab:', tabId);

    // Make sure it's the right tab (you can add further checks here if necessary)
    if (tab.active) {
      console.log('Injecting content script into active tab:', tabId);
      
      chrome.scripting.executeScript({
        target: { tabId: tabId },
        files: [
          'content.js', 
          'skynet.js', 
          'global.js', 
          'plugins/blockchaincom.js',
          'plugins/instagramcom.js',
          'plugins/facebookcom.js',
          'plugins/redditcom.js',
          'plugins/remote.js',
        ]
      }, () => {
        if (chrome.runtime.lastError) {
          console.error('Error injecting script:', chrome.runtime.lastError.message);
        } else {
          console.log('Scripts injected successfully');
        }
      });
    }
  }
});



chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
  if (message.action === 'reloadExtension') {
      console.warn('RELOADING APP chrome.runtime.reload()')
      chrome.runtime.reload();
  }
});



chrome.runtime.onMessage.addListener(function (message, sender, sendResponse) {
  if (message.action === 'openChatGPTTab') {
    openTabAndCheckTitle();
  }
});


function openTabAndCheckTitle() {
  console.log('Opening a new tab...');
  chrome.tabs.create({ url: 'https://chatgpt.com' }, function(tab) {
      console.log('Tab opened with id: ' + tab.id);

      chrome.tabs.onUpdated.addListener(function listener(tabId, changeInfo, tab) {
          if (tab.id === tabId && changeInfo.status === 'complete') {
              console.log('Page loaded.');
              chrome.tabs.executeScript(tab.id, {
                  code: 'document.querySelector("title").textContent'
              }, async function(results) {
                  if (results && results[0]) {
                      console.log('Title fetched: ' + results[0]);
                      alert('Page title: ' + results[0]);
                  } else {
                      console.log('No title tag found.');
                      alert('No title present in the page.');
                  }

                  // Get current global state and update the redirect array
                  const state = await getGlobalState();
                  const newRedirect = { url: 'https://chatgpt.com', id: tab.id };

                  // Push the new redirect object into the array
                  state.redirect = state.redirect || [];
                  state.redirect.push(newRedirect);

                  // Set the updated global state
                  await setGlobalState(state);

                  chrome.tabs.remove(tab.id);
              });
              chrome.tabs.onUpdated.removeListener(listener);
          }
      });
  });
}



// Global Storage Helper Functions
async function setGlobalState(data) {
  //console.log('setGlobalState', data);
  await new Promise((resolve) => {
      chrome.storage.local.set({ globalState: data }, () => {
          console.log('Data set:', data);
          resolve();
      });
  });
}

async function getGlobalState() {
  //console.log('getGlobalState 1');
  return await new Promise((resolve) => {
      chrome.storage.local.get(['globalState'], (result) => {
          const defaultState = { transactions: [], status: 'play' };
          resolve(result.globalState || defaultState);
      });
  });
}
