// List of allowed domains
allowedDomains = [
  'suno.com',
  'chatgpt.com',
  'openai.com',
  'sora.com',
  'blockchain.com',
  'facebook.com',
  'instagram.com',
  'distrokid.com',
  'spotify.com',
  'reddit.com'
];

// Get the current domain name
currentDomain = window.location.hostname;

// Check if the domain name is in the list of allowed domains
if (allowedDomains.includes(currentDomain)) {
  // // Add a class to the body with the sanitized website name (e.g., 'chatgptcom')
  // if (typeof document.body !== 'null') {
  //   document.body.classList.add(currentDomain.replace('.', '').toLowerCase());
  // }

  // Add the 'nexus-enabled' class to the <html> element
  document.documentElement.classList.add('nexus-enabled');

  console.log(`Nexus active on ${currentDomain}`);
}

// Global Storage Helper Functions
async function setGlobalState(data) {
    //console.log('setGlobalState', data);
    await new Promise((resolve) => {
        chrome.storage.local.set({ globalState: data }, () => {
            console.log('Data set:', data);
            resolve();
        });
    });
}

async function getGlobalState() {
    //console.log('getGlobalState');
    return await new Promise((resolve) => {
        chrome.storage.local.get(['globalState'], (result) => {
            const defaultState = { transactions: [], status: 'play' };
            resolve(result.globalState || defaultState);
        });
    });
}

function getState() {
    let state = JSON.parse(localStorage.getItem('state')) || { status: 'paused', events: [], tickets: [] };
    return state;
}

function setState(state) {
    localStorage.setItem('state', JSON.stringify(state));
}










// Function to create a ticket
function createTicket(errorData) {
  let state = getState();

  // Ensure state.tickets is an object
  if (!state.tickets || typeof state.tickets !== "object") {
      state.tickets = {};
  }

  // Remove any null or undefined tickets before proceeding
  Object.keys(state.tickets).forEach(ticketId => {
      if (state.tickets[ticketId] === null || state.tickets[ticketId] === undefined) {
          delete state.tickets[ticketId];
      }
  });

  // Save cleaned state
  setState(state);

  const filePath = errorData.sourceURL || "Unknown File";
  const errorMessage = errorData.message || "No message available";
  const stackTrace = errorData.stack || "No stack trace available";

  // Generate a unique ticket ID
  let nextId = Object.keys(state.tickets).length + 1;
  let ticketId = `NEXUS-${String(nextId).padStart(3, "0")}`;

  let newTicket = {
      id: ticketId,
      status: "open",
      type: "bug",
      summary: `console.log ${errorMessage} in ${filePath}`, // Short summary
      description: `Error in file: ${filePath}`,
      logs: [
          "Opened the ticket...",
          `Error Message: ${errorMessage}`,
          `Stack Trace: ${stackTrace}`
      ]
  };

  // Prevent duplicate tickets
  const exists = Object.values(state.tickets).some(ticket => ticket.summary === newTicket.summary);
  if (!exists) {
      state.tickets[ticketId] = newTicket; // Store in object
      setState(state);
      console.log(`✅ Created ticket: ${JSON.stringify(newTicket, null, 2)}`, state);
  } else {
      console.log(`⚠️ Ticket already exists for this error: ${filePath}`);
  }
}

































// Override console.error to capture errors while keeping normal logging
(function () {

    document.addEventListener('keydown', function(event) {
        if (event.key === "Enter" && event.shiftKey) {
            let promptTextarea = document.querySelector("#prompt-textarea");
            if (promptTextarea) {
                let parent = promptTextarea.closest('[class^="_prosemirror"]');
                if (parent) {
                    parent.scrollTop += 12; // Scroll 12px down each time
                    console.log("Scrolled down by 12px in parent with class starting with _prosemirror.");
                }
            }
        }
    });


    const originalError = console.error;

    console.error = function (...args) {
        originalError.apply(console, args);

        let errorData = {
            message: args.map(a => (typeof a === "object" ? JSON.stringify(a) : a)).join(" "),
            sourceURL: (new Error()).stack?.split('\n')[1]?.trim() || "Unknown source",
            stack: (new Error()).stack || "No stack trace available"
        };

        createTicket(errorData);
    };

})();

// Capture global JavaScript runtime errors
window.addEventListener("error", (event) => {
    let errorData = {
        message: event.message,
        sourceURL: event.filename,
        stack: event.error?.stack || "No stack trace available"
    };
    
    createTicket(errorData);
});

// Capture unhandled promise rejections
window.addEventListener("unhandledrejection", (event) => {
    let errorData = {
        message: event.reason.message || "Unhandled Promise Rejection",
        sourceURL: event.reason.stack?.split('\n')[1]?.trim() || "Unknown source",
        stack: event.reason.stack || "No stack trace available"
    };

    createTicket(errorData);
});
