console.log('GLOBAL LOADED');
let soundPlaying = false;



function playSound(type) {
    let state = getState()
    if (state.mute) {
        return;
    }
    if (soundPlaying) return; // Avoid overlapping sounds
    soundPlaying = true;

    const context = new (window.AudioContext || window.webkitAudioContext)();

    if (type === 'initializing_systems' || type === 'new_project' || type === 'testing') {
        const audio = new Audio(); // Create a new audio element

        // Map new sound types to mp3 files
        const soundMap = {
            'initializing_systems': 'sounds/initializing_systems.mp3',
            'new_project': 'sounds/new_project.mp3',
            'testing': 'sounds/testing.mp3'
        };

        audio.src = soundMap[type];
        audio.play();

        // Listen for the sound to end before allowing another to play
        audio.onended = () => {
            soundPlaying = false;
        };

        // In case of error, reset soundPlaying to allow future sounds
        audio.onerror = () => {
            console.error(`Error playing sound for type: ${type}`);
            soundPlaying = false;
        };

        return;
    }

    const oscillator = context.createOscillator();

    let frequency = 200;

    if (type === 'wub') {
        frequency = 120;
    }

    if (type === 'dub') {
        frequency = 160;
    }

    if (type === 'beep') {
        frequency = 180;
    }

    if (type === 'alert') {
        frequency = 400;
    }

    if (type === 'chirp') {
        frequency = 1200;
    }

    oscillator.type = 'sine'; // Waveform type
    oscillator.frequency.setValueAtTime(frequency, context.currentTime); // Frequency in Hz (440 Hz = A4)
    oscillator.connect(context.destination);
    oscillator.start();

    setTimeout(() => {
        oscillator.stop();
        soundPlaying = false;
    }, 100); // Play for 1 second
}




// Infinite loop using setInterval
setInterval(() => {
    let state = getState(); // Retrieve state

    if (state.alert > 0 && !state.mute) { // Check if state.alert > 0
        if(!state.playSound) {
            state.playSound = 'alert'
        }
        playSound(state.playSound); // Play sound
        state.alert--; // Decrement state.alert
        setState(state)
        console.log(`Alert decreased to: ${state.alert}`);
    }
    else {
        //console.log('alert off')
    }
}, 100); // Run the loop every 100 milliseconds


    



(function multimedia() {
    console.log('MULTIMEDIA LOADED');
    const CHECK_INTERVAL = 1000; // Check every 1 second
    let soundPlaying = false; // Track if sound is currently playing

    function checkForSvgAndMeta() {
        // Check for the target SVG path
        const targetPath = document.querySelector('path[d="M3.8 9.603a8.248 8.248 0 0 1 16.399 0l.674 6.18A2 2 0 0 1 18.885 18h-2.043a5.002 5.002 0 0 1-9.685 0H5.115a2 2 0 0 1-1.988-2.217zM9.273 18a3 3 0 0 0 5.456 0zM12 4.25a6.25 6.25 0 0 0-6.21 5.57L5.114 16h13.77l-.674-6.18A6.25 6.25 0 0 0 12 4.25"]');

        if (!state) {
            state = getState();
        }

        if (!state.playSound) {
            state.playSound = null;
        }


        if (!targetPath && state.application === "ChatGPT") {
            console.log('Target SVG path found on the page. Playing warning sound.');
            //playSound("beep");
        }


        setTimeout(checkForSvgAndMeta, CHECK_INTERVAL);
    }



    function checkForGeneration() {
        let stopButton = document.querySelector('button[data-testid="stop-button"]');

        // Check for the send button every second and click it when found
        const intervalId = setInterval(() => {
            let state = getState();
            const stopButton = document.querySelector('button[data-testid="stop-button"]');
            if (stopButton && state.application != 'sora') {
                if (!state.mute) {
                    state.playSound = "chirp";
                    setState(state);
                }
                clearInterval(intervalId); // Stop the loop after the button is clicked
            } else {
                if (state.application === 'chatgpt') {
                    state.playSound = null;
                    setState(state);
                    if (state.debug) {
                        console.warn('Stop button not found, retrying...');
                    }
                    
                }
            }
        }, 1000); // Repeat every 1 second

    }

    function skynet(state) {
        if (state.debug) {
            console.log('skynet has loaded | typeof state: ' + typeof state);
            logEvent({
                type: 'system',
                subtype: 'skynet',
                tags: ['skynet'],
                message: 'skynet has loaded | typeof state: ' + typeof state,
                status: 'ready',
            });
        }

        // Queue for handling actions sequentially with waits
        let actionQueue = [];
        let isProcessing = false;
    
        // Function to process the next action in the queue
        const processQueue = () => {
            if (isProcessing || actionQueue.length === 0) return;
    
            isProcessing = true;
            const { action, delay } = actionQueue.shift();
    
            setTimeout(() => {
                action();
                isProcessing = false;
                processQueue(); // Process the next action in the queue
            }, delay);
        };
    
        // Helper function to add actions with delays to the queue
        const enqueueAction = (action, delay = 0) => {
            actionQueue.push({ action, delay });
            processQueue();
        };
    
        // Start the main loop
        const intervalId = setInterval(() => {
            let state = getState();
    
            // Handle play sound logic
            if (state.playSound && !state.mute) {
                enqueueAction(() => {
                    if (state.debug) {
                        logEvent({
                            type: 'system',
                            subtype: 'skynet',
                            tags: ['sound'],
                            message: 'playSound has played a sound',
                            status: 'play',
                        });
                    }

                    playSound(state.playSound);
                    console.log(`play sound ended [${state.playSound}] state:`, state);
                    state.playSound = null;
                    setState(state);
                }, state.wait || 0);
            }

            if (state.application == 'suno' && state.status != 'paused') {
                // Part 1: Find and click the button with "Download File" text
                enqueueAction(() => {
                    // Find the div with class 'fixed'
                    const fixedDiv = document.querySelector('.fixed');
                    if (fixedDiv) {
                        // Find a button with "Download File" text that is not disabled
                        const downloadButton = Array.from(fixedDiv.querySelectorAll('button'))
                            .find(button => button.innerText === 'Download File' && !button.disabled);
                        // Click the button if it exists and is not disabled
                        if (downloadButton) {
                            downloadButton.click();
                        }
                    }
                }, state.wait || 2000);

                // Part 2: Navigate and trigger click on specific buttons
                enqueueAction(() => {
                    // Find the GridList
                    const gridList = document.querySelector('.react-aria-GridList');

                    if (gridList) {
                        // Find the currently focused GridListItem (tabindex="0")
                        const currentItem = gridList.querySelector('.react-aria-GridListItem[tabindex="0"]');
                        if (currentItem) {
                            // Get all siblings in the list
                            const siblings = Array.from(gridList.querySelectorAll('.react-aria-GridListItem'));
                            const currentIndex = siblings.indexOf(currentItem);
                            const nextItem = siblings[currentIndex + 1]; // Move to the next item in the list

                            if (nextItem) {
                                // Find the div with aria-label="Play Song" inside the next item
                                const playSongDiv = nextItem.querySelector('div[aria-label="Play Song"]');
                                if (playSongDiv) {
                                    // Find the div next to it (sibling div after Play Song div)
                                    const nextToPlaySongDiv = playSongDiv.nextElementSibling;
                                    if (nextToPlaySongDiv) {
                                        // Find the last button inside the sibling div
                                        console.log('nextToPlaySongDiv', nextToPlaySongDiv)
                                        const lastButton = nextToPlaySongDiv.querySelector('button:last-of-type');
                                        if (lastButton) {
                                            // Trigger a click on the last button
                                            lastButton.click();
                                            lastButton.setAttribute('data-state', 'open');
                                            console.log('lastButton:', lastButton);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }, state.wait || 2000);

            }
    
            // Handle application logic
            if (state.application === 'sora') {
    
                if (state.status === 'paused') {
                    console.log('SORA paused:');
                }
    
                if (state.status === 'play') {
                    if (!state.mute) {
                        state.playSound = null;
                        setState(state);
                    }

    
                    // rEMIX I THING
                    enqueueAction(() => {
                        console.log('2 seconds have passed.');
    
                        // Find and click the "Create" button
                        const createButtons = document.querySelectorAll('button');
                        for (const createButton of createButtons) {
                            if (createButton.textContent.trim() === 'Create') {
                                console.log('CREATE CLICKED');
                                createButton.click();
                                break; // Exit loop after clicking the button
                            }
                        }
                    }, state.wait || 2000);
    
                    enqueueAction(() => {

                        // Handle textarea interaction
                        let textarea = document.querySelector('textarea[placeholder^="Describe"]');


                        if (textarea) {
                            console.log('state_here', state)
                            textarea.value = state && typeof state.prompt !== 'undefined' ? state.prompt : state.prompt;
                            if (document.querySelector('.prompt-textarea')) {
     
                                
                               
                                document.querySelector(".prompt-textarea").parentNode.appendChild(newButton);
                                console.log('promptFromModal', promptFromModal);

                                if (promptFromModal && !state.prompt) {
                                    textarea.value = promptFromModal;
                                    state.prompt = promptFromModal;
                                }

                                console.log('state', state);
                            }

                            textarea.value = state.prompt || "Default prompt text";

                            // Trigger an input event to ensure any listeners react
                            const inputEvent = new Event('input', { bubbles: true, cancelable: true });
                            textarea.dispatchEvent(inputEvent);
                            console.log('Textarea filled with value:', textarea.value);
                        } else {
                            console.warn('Textarea with specified placeholder not found.');
                        }
    

                        // Find and click the "Remix" button
                        const remixButtons = document.querySelectorAll('button');
                        for (const remixButton of remixButtons) {
                            if (remixButton.textContent.trim() === 'Remix') {
                                console.log('REMIX CLICKED');
                                remixButton.click();
                                break; // Exit loop after clicking the button
                            }
                        }
                    }, state.wait || 2000);
    

    
                    enqueueAction(() => {
                        // Find and click the "Create video" or "Remix" button
                        const buttons = document.querySelectorAll('button');
                        let createVideoButton = null;

                        // Handle textarea interaction
                        let textarea = document.querySelector('textarea[placeholder^="Describe"]');
        
                        if (textarea) {
                            textarea.value = state.prompt;
                            if (document.querySelector('.prompt-textarea')) {
                                console.log('promptFromModal', promptFromModal);

                                if (promptFromModal && !state.prompt) {
                                    textarea.value = promptFromModal;
                                    state.prompt = promptFromModal;
                                }

                                console.log('state', state);
                            }

                            textarea.value = state.prompt || "Default prompt text";

                            // Trigger an input event to ensure any listeners react
                            const inputEvent = new Event('input', { bubbles: true, cancelable: true });
                            textarea.dispatchEvent(inputEvent);
                            console.log('Textarea filled with value:', textarea.value);
                        } else {
                            console.log('Textarea with specified placeholder not found. Open a project?');
                        }
    
                        buttons.forEach(button => {
                            if (button.textContent.trim() === 'Create video') {
                                createVideoButton = button;
                            }
                            if (button.textContent.trim() === 'Remix') {
                                createVideoButton = button;
                            }
                            if (button.textContent.trim() === 'Create') {
                                createVideoButton = button;
                            }
                        });
    
                        if (createVideoButton) {
                            console.log('Clicked the "Create video" button.');
                            createVideoButton.click();
                        } else {
                            console.warn('Button with text "Create video" not found.');
                        }
                    }, state.wait || 2000);
                }
            }
        }, 500); // Repeat every 500ms
    }
    






    console.log('SKYNET MONITORING SYSTEM ACTIVATED 🤖.');
            
    let state = getState();

    // Ensure .tickets exists
    if (!state.tickets) {
        state.tickets = {};
    }

    // Ensure state.title exists, default to 'NEXUS-001' if not
    const title = state.title || 'NEXUS-001';

    // Ensure .logs exists for the specified title
    if (!state.tickets[title]) {
        state.tickets[title] = {};
    }

    if (!state.tickets[title].logs) {
        state.tickets[title].logs = [];
    }

    // Push the log message
    //state.tickets[title].logs.push("Initiated Ticket Log");



    // Get the hostname of the current website
    let hostname = window.location.hostname;

    // Extract the domain name before ".com" or remove all periods if ".com" is not present
    let applicationName;
    if (hostname.includes('.com')) {
        applicationName = hostname.split('.com')[0].split('.').pop().toLowerCase();
    } else {
        applicationName = hostname.replace(/\./g, '').toLowerCase();
    }

    console.log("HEY", applicationName);
    // Update the state object
    state.application = applicationName;
    if (!state.status) {
        console.log('status set to start')
        state.status = 'start';
    }

    if (!Array.isArray(state.events)) {
        state.events = [];
    }

    // Save the updated state
    setState(state);
    console.log('START STATE:', state);

    state.playSound = null;
    checkForSvgAndMeta();
    checkForGeneration();
    skynet(state);





})();
