console.log('Q VERSION 3.4');
console.log('SKYNET LOADED');
let TOKEN_MAX = 300000;
let TOKEN_MAX_HALF = 700000;
let TOKEN_OVERMAX = 1000000;


// Function to log events into localStorage
function logEvent({ type, subtype, tags, message, status }) {
    let state = getState()
    const event = {
        uuid: crypto.randomUUID(),
        type,
        subtype,
        tags,
        message,
        status,
        timestamp: new Date().toISOString(),
    };

    if (Array.isArray(state.events)) {
        state.events.push(event);
    }
    else {
        console.log('🔴 logEvent ERROR or empty', event);
        state.events = [];
    }

    setState(state)
    console.log('🔵 Event logged {event,state}:', {event,state});
}

// Global Storage Helper Functions
function globalSet(data, callback) {
    chrome.storage.local.set(data, callback || (() => console.log('Data set:', data)));
}

function globalGet(key, callback) {
    chrome.storage.local.get([key], (result) => {
        callback(result[key] || null);
    });
}

function getAllFlags() {
    let state = getState();
    if (!state.flags || !Array.isArray(state.flags)) return [];

    return state.flags.map(flag => flag.id);
}


function checkFlag(uuid) {
    let state = getState();
    if (!state.flags || !Array.isArray(state.flags)) return false;

    return state.flags.some(flag => flag.id === uuid);
}

function getFlag(uuid) {
    let state = getState();
    if (!state.flags || !Array.isArray(state.flags)) return null;

    let flag = state.flags.find(flag => flag.id === uuid);
    return flag ? flag.emoji : null;
}




// Skynet OS
(function () {


    // Initialize localStorage state if not set
    if (!localStorage.getItem('state')) {
        localStorage.setItem(
            'state',
            JSON.stringify({ currentProject: '1', events: [] })
        );
    }


    // State variable for the current project
    let currentProject = JSON.parse(localStorage.getItem('state')).currentProject;


    // Check if the container already exists
    let existingContainer = document.getElementById('skynet-container');

    const container = document.createElement('div');
    container.id = 'skynet-container';

    const menu = document.createElement('div');
    menu.id = 'menu';

    const mainContent = document.createElement('div');
    mainContent.id = 'main-content';

    container.appendChild(menu);
    container.appendChild(mainContent);

    if (existingContainer) {
        existingContainer.replaceWith(container);
    } else {
        document.body.appendChild(container);
    }


    // Modal handler
    function showModal(title, contentGenerator) {
        const existingModal = document.querySelector('.nexus-modal');
        if (existingModal) existingModal.remove();

        const modal = document.createElement('div');
        modal.id = 'modal-' + title.toLowerCase().replace(/\s+/g, '-').replace(/_/g, '-').replaceAll('[', '').replaceAll(']', ''); // Modification for ticket TICKET-NEXUS-001
        modal.classList.add('nexus-modal');

        const modalTitle = document.createElement('h2');
        modalTitle.textContent = title;
        modal.appendChild(modalTitle);

        const closeButton = document.createElement('button');
        closeButton.classList.add('close-button');
        closeButton.textContent = 'Close';
        closeButton.addEventListener('click', () => {            
            let state = getState();
            state.alert = 1;
            setState(state)
            modal.remove()
        }

    );
        modal.appendChild(closeButton);

        contentGenerator(modal);

        document.body.appendChild(modal);
    }

    // Create menu buttons
    const menuIcons = [
        { emoji: '❓', title: 'Tools', action: showToolsModal },
        { emoji: '💻', title: 'Terminal', action: showTerminalModal },
        { emoji: '➕', title: 'New', action: showNewModal },
        { emoji: '📁', title: 'Files', action: showFilesModal },
        { emoji: '⚙️', title: 'Configuration', action: showConfigurationModal },
        { emoji: '❓', title: 'Documentation', action: showDocumentationModal },
        { emoji: '🤖', title: 'Automate', action: awaitUser },
        { emoji: '▶️', title: 'Play', action: playState },
        { emoji: '⏸️', title: 'Pause', action: pauseState },
        { emoji: '🔇', title: 'Mute', action: muteState },
        { emoji: '❌', title: 'Restart', action: () => restartApp() },
    ];

    
    

    menuIcons.forEach(icon => {
        const button = document.createElement('button');
        button.textContent = icon.emoji;
        button.title = icon.title;
    
        // Add 'menu-button' class
        button.className = 'menu-button';
    
        // Generate an ID by converting title to lowercase, replacing spaces and underscores with dashes
        button.id = 'menu-' + icon.title.toLowerCase().replace(/\s+/g, '-').replace(/_/g, '-');
    
        button.addEventListener('click', icon.action);
        menu.appendChild(button);
    }
);
    






















// Helper function to send a prompt
function sendPrompt(prompt) {
    // Find the prompt text area and input the prompt data
    const promptTextarea = document.querySelector('#prompt-textarea');
    let state = getState();

    if (promptTextarea) {
        // Check if prompt length is greater than the token overmax
        if (prompt.length > TOKEN_OVERMAX) {
            const halfTokenMax = TOKEN_OVERMAX / 2;  // Split prompt into halves based on TOKEN_OVERMAX
            const chunkCount = Math.ceil(prompt.length / halfTokenMax);
            const chunks = [];

            // Split the prompt into chunks of size TOKEN_OVERMAX / 2
            for (let i = 0; i < chunkCount; i++) {
                let chunk = prompt.slice(i * halfTokenMax, (i + 1) * halfTokenMax);

                // If it's not the last chunk, append a waiting message
                if (i < chunkCount - 1) {
                    chunk += " THIS IS A CHUNKED PROMPT AS IT WAS TOO LONG. WAIT FOR ME TO SAY READY AND WE ARE ON THE FINAL CHUNK. JUST RESPOND WITH OK FOR NOW";
                } else {
                    // For the last chunk, prepend a final execution message
                    chunk = "THIS IS A CHUNKED PROMPT BUT THIS IS THE FINAL CHUNK EXECUTE THE WHOLE PROMPT AFTER THIS " + chunk;
                }

                chunks.push(chunk);
            }

            // Save the chunks in the state and lock the process
            state.chunks = chunks;
            state.status = 'locked';
            state.locked = chunks.length;
            setState(state);

            let currentChunkIndex = 0;

            // Function to process and send each chunk
            function processNextChunk() {
                if (currentChunkIndex < chunks.length) {
                    const chunk = chunks[currentChunkIndex];

                    // Wait for the send button to be available
                    const intervalId = setInterval(() => {
                        const sendButton = document.querySelector('button[data-testid="send-button"]');
                        if (sendButton) {
                            // Set the prompt textarea to the current chunk and click send
                            promptTextarea.textContent = chunk;
                            sendButton.click();

                            // Increment the chunk index and stop the interval
                            currentChunkIndex++;
                            clearInterval(intervalId);

                            // Update the state after each chunk is sent
                            state.locked = chunks.length - currentChunkIndex;
                            setState(state);

                            // If all chunks are processed, set the status back to play
                            if (currentChunkIndex === chunks.length) {
                                state.status = 'play';
                                setState(state);
                            } else {
                                // Continue to the next chunk after a small delay
                                setTimeout(processNextChunk, 1000);
                            }
                        }
                    }, 1000);
                }
            }

            // Start processing the chunks
            processNextChunk();
        } else {
            promptTextarea.textContent = state.prompt;


            // If the prompt doesn't exceed the max token length, send it as usual
            setTimeout(() => {
                const sendButton = document.querySelector('button[data-testid="send-button"]');
                if (sendButton) {
                    sendButton.click();
                } else {
                    console.warn('Send button not found.');
                }
            }, 1500);
        }
    } else {
        // Alert if prompt text area not found
        state.alert = 1;
        setState(state);
        return;
    }
}





























async function getResponse() {
    console.log('Waiting 1 second before starting...');
    await new Promise(resolve => setTimeout(resolve, 1000)); // Wait for 1 second

    console.log('GENERATING...');
    return new Promise((resolve, reject) => {
        const intervalId = setInterval(() => {
            const articles = document.querySelectorAll('article');
            if (articles.length > 0) {
                const lastArticle = articles[articles.length - 1];
                const copyButton = lastArticle.querySelector('[data-testid="copy-turn-action-button"]');

                if (copyButton) {
                    console.log('Copy button found in the last article.');

                    // Wait for 1 second to ensure all content is loaded
                    setTimeout(() => {
                        let articleText = lastArticle.textContent || lastArticle.innerText;
                        if (articleText) {
                            // TODO: this might need work
                            // console.log('Article Text:', articleText);
                            articleText = articleText.match(/({.*})/s)?.[1].trim().replace(/4o$/, '').replace('ChatGPT said:ChatGPTjsonCopyEdit', '');
                            // console.log('Article Text:', articleText);

                            clearInterval(intervalId);
                            resolve(articleText);
                        } else {
                            console.log('No text content found after waiting.');
                        }
                    }, 1000); // Delay of 1 second
                } else {
                    console.log('GENERATING... Copy button not found in the last article.');
                }
            } else {
                console.log('No articles found on the page.');
            }
        }, 1000); // Check every second
    });
}


function restartApp() {
    chrome.runtime.sendMessage({action: 'reloadExtension'});
    location.reload()
}

// Function to render the menu buttons
const renderMenuButtons = () => {
    const menuContainer = document.getElementById('menu'); // Assuming a container with id "menu"
    menuContainer.innerHTML = ''; // Clear the container
    menuIcons.forEach(({ emoji, title, action }) => {
        const button = document.createElement('button');
        button.textContent = `${emoji}`;
        if (title == 'Restart') {
            // TODO: KEEP THIS ITS BASICALLY hook_menu_alter
        }
        
        button.title = title;
    
        // Add 'menu-button' class
        button.className = 'menu-button';
    
        // Generate an ID by converting title to lowercase, replacing spaces and underscores with dashes
        button.id = 'menu-' + title.toLowerCase().replace(/\s+/g, '-').replace(/_/g, '-');
    
        button.onclick = action;
        menuContainer.appendChild(button);
    });
};



function showTerminalModal() {
    let state = getState();
    state.alert = 1;
    setState(state);

    showModal('[Terminal]', (modal) => {
        // Create the iframe for ChatGPT
        const terminalIframe = document.createElement('iframe');
        terminalIframe.src = 'https://chat.openai.com/';
        terminalIframe.style.width = '100%';
        terminalIframe.style.height = '400px';
        terminalIframe.style.minHeight = '400px'; // Added min-height
        terminalIframe.style.border = 'none';

        // Create the prompt input
        const promptDiv = document.createElement('div');
        const promptLabel = document.createElement('label');
        promptLabel.textContent = 'Prompt:';
        const promptInput = document.createElement('textarea');
        promptInput.placeholder = 'Enter your prompt here...';
        promptInput.style.width = '100%';
        promptInput.style.height = '80px';
        promptInput.classList.add('terminal-prompt-input');

        // Add prompt label and input to the div
        promptDiv.appendChild(promptLabel);
        promptDiv.appendChild(promptInput);

        // Create the response field
        const responseDiv = document.createElement('div');
        const responseLabel = document.createElement('label');
        responseLabel.textContent = 'Response:';
        const responseField = document.createElement('textarea');
        responseField.style.width = '100%';
        responseField.style.height = '120px';
        responseField.classList.add('terminal-response-field');
        responseField.readOnly = true;

        // Add response label and field to the div
        responseDiv.appendChild(responseLabel);
        responseDiv.appendChild(responseField);

        // Create the "Send Prompt" button
        const sendPromptButton = document.createElement('button');
        sendPromptButton.textContent = 'Send Prompt';
        sendPromptButton.classList.add('send-prompt-button');
        sendPromptButton.addEventListener('click', () => {
            console.log('Send Prompt button clicked');
            console.log('Prompt value:', promptInput.value);

            const iframeDocument = terminalIframe.contentDocument || terminalIframe.contentWindow.document;
            const promptTextarea = iframeDocument.querySelector('#prompt-textarea');
            const sendButton = iframeDocument.querySelector('button[data-testid="send-button"]');

            if (promptTextarea) {
                promptTextarea.value = promptInput.value;
                const inputEvent = new Event('input', { bubbles: true });
                promptTextarea.dispatchEvent(inputEvent);
                console.log('Prompt dispatched to iframe');
                setTimeout(() => {
                    sendButton?.click();
                    console.log('Send button clicked in iframe');
                }, 500);
            } else {
                console.warn('Prompt text area not found inside the iframe.');
            }
        });

        // Create the "Get Response" button
        const getResponseButton = document.createElement('button');
        getResponseButton.textContent = 'Get Response';
        getResponseButton.classList.add('get-response-button');
        getResponseButton.addEventListener('click', async () => {
            console.log('Get Response button clicked');
            const iframeDocument = terminalIframe.contentDocument || terminalIframe.contentWindow.document;

            const waitForResponse = async () => {
                return new Promise((resolve) => {
                    const intervalId = setInterval(() => {
                        const articles = iframeDocument.querySelectorAll('article');
                        if (articles.length > 0) {
                            const lastArticle = articles[articles.length - 1];
                            const articleText = lastArticle.textContent || lastArticle.innerText;
                            if (articleText) {
                                clearInterval(intervalId);
                                resolve(articleText);
                            }
                        }
                    }, 1000);
                });
            };

            console.log('Waiting for response...');
            const response = await waitForResponse();
            console.log('Response received:', response);

            // Update the response field
            responseField.value = response;
        });

        // Append elements to the modal
        modal.appendChild(terminalIframe);
        modal.appendChild(promptDiv);
        modal.appendChild(responseDiv);
        modal.appendChild(sendPromptButton);
        modal.appendChild(getResponseButton);
    });
}






































// Fetch and update the "Tools" icon dynamically
const updateToolsIcon = (state) => {
    
    fetch('http://localhost:3666/test')
        .then((res) => res.json())
        .then((data) => {
            // Update the emoji based on the connection status
            menuIcons[0].emoji = data.status === 'ok' ? `✅ [${state.status}]` : '❌';
            renderMenuButtons(); // Re-render the menu buttons to reflect the change
        })
        .catch(() => {
            // Update the emoji to ❌ if fetch fails
            menuIcons[0].emoji = '❌';
            renderMenuButtons(); // Re-render the menu buttons to reflect the change
        });
};


// Refactored showToolsModal Function
function showToolsModal() {
    let state = getState();
    state.alert = 1;
    setState(state)

    showModal('[Test Connection]', (modal) => {
        // Status indicator for connection
        const statusIndicator = document.createElement('div');
        statusIndicator.classList.add('status-indicator');
        statusIndicator.textContent = 'Connection: ❌';

        // Test Connection button
        const testButton = document.createElement('button');
        testButton.classList.add('test-button');
        testButton.textContent = 'Tools Local Connection';
        testButton.addEventListener('click', () => {
            let state = getState();
            state.alert = 1;
            setState(state)
            fetch('http://localhost:3666/test')
                .then((res) => res.json())
                .then((data) => {
                    statusIndicator.textContent = data.status === 'ok' ? 'Connection: ✅' : 'Connection: ❌';
                })
                .catch(() => {
                    statusIndicator.textContent = 'Connection: ❌';
                });
        });

        // Automatically test the connection when the modal opens
        setTimeout(() => testButton.click(), 0);

        // Tools AI Connection button
        const testAIButton = document.createElement('button');
        testAIButton.classList.add('test-ai-button');
        testAIButton.textContent = 'Tools AI Connection';
        testAIButton.addEventListener('click', () => {
            let state = getState();
            state.alert = 1;
            setState(state)
            try {
                // Gather localStorage data
                let localStorageData = '';
                for (let i = 0; i < localStorage.length; i++) {
                    const key = localStorage.key(i);
                    const value = localStorage.getItem(key);
                    localStorageData += `${key}: ${value}\n`;
                }

                // Send the prompt
                sendPrompt(localStorageData);

                // Get and display the AI response
                getResponse((articleContent) => {
                    const resultDisplay = document.createElement('div');
                    resultDisplay.classList.add('result-display');
                    resultDisplay.textContent = `AI Response:\n${articleContent}`;
                    modal.appendChild(resultDisplay);
                });
            } catch (error) {
                // alert('An error occurred: ' + error.message);
                let state = getState();
                state.alert = 1;
                setState(state)
            }
        });

        // Notes Section
        const notesFieldset = document.createElement('fieldset');
        notesFieldset.classList.add('notes-fieldset');

        const notesLegend = document.createElement('legend');
        notesLegend.textContent = 'Install';
        notesFieldset.appendChild(notesLegend);

        const notesTextarea = document.createElement('textarea');
        notesTextarea.classList.add('notes-textarea');
        notesTextarea.placeholder = 'Enter your notes here...';

        // Load existing notes from global storage
        globalGet('notes', (notes) => {
            notesTextarea.value = notes || 'sudo service nginx stop && sudo node ~/sites/waypoints/nexus/scripts/q_chrome_extension/server.js';
        });

        const saveNotesButton = document.createElement('button');
        saveNotesButton.classList.add('save-notes-button');
        saveNotesButton.textContent = 'Save to Global';
        saveNotesButton.addEventListener('click', () => {
            const notes = notesTextarea.value;
            globalSet({ notes });
            // alert('Notes saved successfully!');
            let state = getState();
            state.alert = 1;
            setState(state)
        });

        const clearGlobalButton = document.createElement('button');
        clearGlobalButton.classList.add('clear-global-button');
        clearGlobalButton.textContent = 'Clear Global';
        clearGlobalButton.addEventListener('click', () => {
            globalSet({}, () => {
                // alert('Global storage cleared!');
                let state = getState();
                state.alert = 1;
                setState(state)
            });
        });

        const viewGlobalButton = document.createElement('button');
        viewGlobalButton.classList.add('view-global-button');
        viewGlobalButton.textContent = 'View Global';
        viewGlobalButton.addEventListener('click', () => {
            chrome.storage.local.get(null, (data) => {
                const globalDisplay = document.createElement('pre');
                globalDisplay.classList.add('global-display');
                globalDisplay.textContent = JSON.stringify(data, null, 2);
                modal.appendChild(globalDisplay);
            });
        });

        const clearLocalStorageButton = document.createElement('button');
        clearLocalStorageButton.classList.add('clear-localstorage-button');
        clearLocalStorageButton.textContent = 'Clear LocalStorage';
        clearLocalStorageButton.addEventListener('click', () => {
            localStorage.clear();
            // alert('LocalStorage cleared!');
            let state = getState();
            state.alert = 1;
            setState(state)
        });

        // Append notes section elements
        notesFieldset.appendChild(notesTextarea);
        notesFieldset.appendChild(saveNotesButton);
        notesFieldset.appendChild(clearGlobalButton);
        notesFieldset.appendChild(viewGlobalButton);

        // Append elements to the modal
        modal.appendChild(statusIndicator);
        modal.appendChild(testButton);
        modal.appendChild(testAIButton);
        modal.appendChild(notesFieldset);
        modal.appendChild(clearLocalStorageButton);
    });
}





































































// Function to display the Files & Tickets modal
function showFilesModal() {
    let state = getState();

    // Ensure state.tickets exists
    if (!state.tickets) {
        state.tickets = {};
        setState(state);
    }

    if (!state.mute) {
        // Set alert and play sound
        state.alert = 1;
        state.playSound = 'wub';
        setState(state);
    }

    // Fetch the filesystem JSON
    fetch('http://localhost/filesystem.json')
        .then(response => response.json())
        .then(filesystem => {
            showModal('[Files]', (modal) => {
                let projects = localStorage.getItem('skynetProjects');
                let tickets = state.tickets || {};

                if (!projects) {
                    projects = [];
                } else if (typeof projects === 'string') {
                    projects = JSON.parse(projects);
                }

                if (typeof projects === 'object' && !Array.isArray(projects)) {
                    projects = [projects];
                }

                if (projects.length === 0 && Object.keys(tickets).length === 0) {
                    modal.textContent = 'No projects or tickets found.';
                    return;
                }

                if (projects.length > 0) {
                    const projectsTitle = document.createElement('h2');
                    projectsTitle.textContent = 'Projects';
                    projectsTitle.classList.add('projects-title', 'title');
                    modal.appendChild(projectsTitle);

                    projects.forEach((project, index) => {
                        const projectContainer = document.createElement('div');
                        projectContainer.classList.add('projects-container');

                        const projectTitle = document.createElement('h3');
                        projectTitle.textContent = `${index + 1}. ${project.title}`;
                        projectTitle.classList.add('clickable');
                        projectTitle.addEventListener('click', () => {
                            console.log(`Loading project: ${project.title}`);
                            state.prompt = project.prompt;
                            setState(state);
                            showNewModal(project);
                        });

                        const deleteButton = document.createElement('button');
                        deleteButton.textContent = 'Delete';
                        deleteButton.classList.add('delete-button');
                        deleteButton.addEventListener('click', () => {
                            projects.splice(index, 1);
                            localStorage.setItem('skynetProjects', JSON.stringify(projects));
                            modal.remove();
                            showFilesModal();
                        });

                        projectContainer.appendChild(projectTitle);
                        projectContainer.appendChild(deleteButton);
                        modal.appendChild(projectContainer);
                    });
                } else {
                    const noProjectsMessage = document.createElement('h4');
                    noProjectsMessage.textContent = 'NO PROJECTS';
                    noProjectsMessage.classList.add('no-projects');
                    modal.appendChild(noProjectsMessage);
                }

                let ticketEntries = Object.entries(tickets).filter(([_, ticket]) => ticket !== null);
                if (state.debug) {
                    console.log('ticketEntries', ticketEntries)
                }

                if (ticketEntries.length > 0) {
                    const ticketsTitle = document.createElement('h2');
                    ticketsTitle.textContent = 'Tickets';
                    ticketsTitle.classList.add('tickets-title', 'title');
                    modal.appendChild(ticketsTitle);

                    ticketEntries.forEach(([ticketId, ticket], index) => {
                        const ticketContainer = document.createElement('div');
                        ticketContainer.classList.add('tickets-container');

                        const ticketTitle = document.createElement('h3');
                        ticketTitle.textContent = `${index + 1}. ${ticketId} - ${ticket.summary} - [${ticket.status}]`;
                        ticketTitle.classList.add('clickable');
                        ticketTitle.addEventListener('click', () => {
                            console.log(`Loading ticket: ${ticketId}`);
                            showNewTicket({ id: ticketId, ...ticket });
                        });

                        const deleteButton = document.createElement('button');
                        deleteButton.textContent = 'Delete';
                        deleteButton.classList.add('delete-button');
                        deleteButton.addEventListener('click', () => {
                            delete tickets[ticketId];
                            setState(state);
                            modal.remove();
                            showFilesModal();
                        });

                        ticketContainer.appendChild(ticketTitle);
                        ticketContainer.appendChild(deleteButton);
                        modal.appendChild(ticketContainer);
                    });
                } else {
                    const noTicketsMessage = document.createElement('h4');
                    noTicketsMessage.textContent = 'NO TICKETS';
                    noTicketsMessage.classList.add('no-tickets');
                    modal.appendChild(noTicketsMessage);
                }

                const filesTitle = document.createElement('h2');
                filesTitle.textContent = 'Local Filesystem';
                filesTitle.classList.add('files-title', 'title');
                modal.appendChild(filesTitle);

                function buildTree(data) {
                    const tree = {};

                    data.forEach(file => {
                        const relativePath = file.path.replace('/home/nick/sites/waypoints/nexus/scripts/q_chrome_extension', '');
                        const parts = relativePath.split('/');
                        let current = tree;

                        parts.forEach((part, index) => {
                            if (!current[part]) {
                                if (index === parts.length - 1) {
                                    current[part] = { ...file };
                                } else {
                                    current[part] = {};
                                }
                            }
                            current = current[part];
                        });
                    });

                    return tree;
                }

                function renderTree(container, tree) {
                    Object.entries(tree).forEach(([key, value]) => {
                        const item = document.createElement('div');
                        item.classList.add('file-item');

                        if (value.link) {
                            const link = document.createElement('a');
                            link.href = value.link;
                            link.textContent = key;
                            link.target = '_blank';
                            link.classList.add('file-link');
                            item.appendChild(link);
                        } else {
                            const folder = document.createElement('div');
                            folder.textContent = `📁 ${key}`;
                            folder.classList.add('folder');
                            folder.classList.add('expanded'); // ✅ Default folders open

                            const subContainer = document.createElement('div');
                            subContainer.classList.add('sub-container');

                            renderTree(subContainer, value);
                            item.appendChild(folder);
                            item.appendChild(subContainer);
                        }

                        container.appendChild(item);
                    });
                }

                const fileTreeContainer = document.createElement('div');
                fileTreeContainer.classList.add('file-tree-container');
                modal.appendChild(fileTreeContainer);

                const treeData = buildTree(filesystem);
                renderTree(fileTreeContainer, treeData);

                const createTicketButton = document.createElement('button');
                createTicketButton.textContent = 'Create Edit Ticket';
                createTicketButton.classList.add('create-ticket-button');
                createTicketButton.addEventListener('click', () => {
                    modal.innerHTML = '';
                    showNewTicket();
                });

                modal.appendChild(createTicketButton);
            });
        })
        .catch(error => {
            console.error('Error loading filesystem.json:', error);
        });
}






// Function to display or edit a ticket
function showNewTicket(selected = {}) {
    let state = getState();

    showModal('[Edit Ticket]', (modal) => {
        const fields = {
            id: selected.id || '',
            summary: selected.summary || '',
            description: selected.description || '',
            steps: selected.steps || [],
            acceptanceCriteria: selected.acceptanceCriteria || [],
            status: selected.status || 'Open',
            priority: selected.priority || 'Medium',
            logs: selected.logs || []
        };

        Object.keys(fields).forEach((key) => {
            const label = document.createElement('label');
            label.textContent = key.charAt(0).toUpperCase() + key.slice(1);
            modal.appendChild(label);

            if (Array.isArray(fields[key])) {
                const textarea = document.createElement('textarea');
                textarea.value = fields[key].join('\n');
                textarea.placeholder = `Enter ${key} (one per line)`;
                textarea.addEventListener('input', (e) => {
                    fields[key] = e.target.value.split('\n').filter(Boolean);
                });
                modal.appendChild(textarea);
            } else if (key === 'status') {
                const select = document.createElement('select');
                ['Open', 'Analysis', 'In Progress', 'Review', 'Toolsing', 'Closed'].forEach((status) => {
                    const option = document.createElement('option');
                    option.value = status;
                    option.textContent = status;
                    option.selected = fields[key] === status;
                    select.appendChild(option);
                });
                select.addEventListener('change', (e) => {
                    fields[key] = e.target.value;
                });
                modal.appendChild(select);
            } else if (key === 'description') {
                const textarea = document.createElement('textarea');
                textarea.value = fields[key];
                textarea.placeholder = `Enter ${key}`;
                textarea.addEventListener('input', (e) => {
                    fields[key] = e.target.value;
                });
                modal.appendChild(textarea);
            } else {
                const input = document.createElement('input');
                input.type = 'text';
                input.value = fields[key];
                input.placeholder = `Enter ${key}`;
                input.addEventListener('input', (e) => {
                    fields[key] = e.target.value;
                });
                modal.appendChild(input);
            }
        });

        // Add Clear Logs Button
        const clearLogsButton = document.createElement('button');
        clearLogsButton.textContent = 'Clear Logs';
        clearLogsButton.style.marginTop = '10px';
        clearLogsButton.style.backgroundColor = '#ff4d4d';
        clearLogsButton.style.color = 'white';
        clearLogsButton.style.border = 'none';
        clearLogsButton.style.padding = '5px 10px';
        clearLogsButton.style.cursor = 'pointer';
        clearLogsButton.addEventListener('click', () => {
            fields.logs = []; // Clear logs array
            console.log('Logs cleared for ticket:', fields.id);

            state.tickets[fields.id] = fields;

            setState(state);

        });

        modal.appendChild(clearLogsButton);

        const saveButton = document.createElement('button');
        saveButton.textContent = 'Save';
        saveButton.style.marginTop = '10px';
        saveButton.addEventListener('click', () => {
            if (fields.id) {
                state.tickets[fields.id] = fields;
                setState(state);
                modal.remove();
                showFilesModal();
            }
            else {
                console.error('click save ticket', fields)
            }

        });

        modal.appendChild(saveButton);
    });
}


























































function showConfigurationModal() {
    let state = getState();
    state.alert = 1;
    setState(state);

    showModal('[Configuration]', (modal) => {
        // Debug Mode Select Dropdown
        const debugLabel = document.createElement('label');
        debugLabel.textContent = 'Debug Mode: ';

        const debugSelect = document.createElement('select');
        
        const debugOptions = ['false', 'true'];
        debugOptions.forEach(option => {
            const optionElement = document.createElement('option');
            optionElement.value = option;
            optionElement.textContent = option === 'true' ? 'Enabled' : 'Disabled';
            debugSelect.appendChild(optionElement);
        });

        debugSelect.value = state.debug || 'false'; // Set initial state (defaults to 'false')

        // When the dropdown changes, update the state
        debugSelect.addEventListener('change', () => {
            let newState = getState();
            newState.debug = debugSelect.value === 'true'; // Update debug mode as boolean
            setState(newState);
        });

        // Restore email and password functionality
        const emailInput = document.createElement('input');
        emailInput.placeholder = 'ChatGPT Email';
        emailInput.value = localStorage.getItem('chatgptEmail') || '';

        const passwordInput = document.createElement('input');
        passwordInput.placeholder = 'ChatGPT Password';
        passwordInput.type = 'password';
        passwordInput.value = localStorage.getItem('chatgptPassword') || '';

        const saveButton = document.createElement('button');
        saveButton.textContent = 'Save';
        saveButton.addEventListener('click', () => {
            localStorage.setItem('chatgptEmail', emailInput.value);
            localStorage.setItem('chatgptPassword', passwordInput.value);

            let newState = getState();
            newState.alert = 1;
            newState.debug = debugSelect.value === 'true'; // Save the debug mode state
            setState(newState);
        });

        // Flags Section
        const flagsContainer = document.createElement('div');
        const flagsTitle = document.createElement('h3');
        flagsTitle.textContent = 'Flags';

        const addFlagInput = document.createElement('input');
        addFlagInput.placeholder = 'Enter Flag Title';

        const addFlagButton = document.createElement('button');
        addFlagButton.textContent = 'Add Flag';

        const flagsTable = document.createElement('table');
        flagsTable.border = '1';

        const FLAG_OPTIONS = [
            '🔴', '🟠', '🟡', '🟢', '🔵', '🟣', '🟤', '⚫', '⚪',
            '🚨', '⚠️', '❗', '❕', '❓', '‼️', '⁉️',
            '🚩', '🏴', '🏁', '🏳️', '🏳️‍🌈', '🏳️‍⚧️', '🏴‍☠️',
            '✅', '❌', '❎', '🛑', '⛔', '🔺', '🔻', '🔸', '🔹', '🔶', '🔷',
            '⬆️', '⬇️', '⬅️', '➡️', '🔄', '🔃', '🔀', '🔁', '🔂',
            '💡', '🔥', '⚡', '💥', '🎯', '💰', '💎', '🛡️', '🚀', '🎉'
        ];

        const emojiSelect = document.createElement('select');
        FLAG_OPTIONS.forEach(emoji => {
            const option = document.createElement('option');
            option.textContent = emoji;
            option.value = emoji;
            emojiSelect.appendChild(option);
        });

        // Default flags
        const defaultFlags = [
            { id: 'flag_1', title: 'Flag 1', emoji: '🚩' },
            { id: 'flag_2', title: 'Flag 2', emoji: '🏴' },
            { id: 'flag_3', title: 'Flag 3', emoji: '⛳' }
        ];

        // Load flags from state or use defaults
        let flags = state.flags || defaultFlags;

        function updateFlagsTable() {
            flagsTable.innerHTML = '<tr><th>Title</th><th>UUID</th><th>Emoji</th><th>Action</th></tr>';
            flags.forEach((flag, index) => {
                let row = document.createElement('tr');
                row.innerHTML = `<td>${flag.title}</td><td>${flag.id}</td><td>${flag.emoji}</td>`;
                
                let deleteButton = document.createElement('button');
                deleteButton.textContent = '❌ Delete';
                deleteButton.style.color = 'red';
                deleteButton.addEventListener('click', () => {
                    flags.splice(index, 1);  // Remove flag from array
                    let newState = getState();
                    newState.flags = flags;
                    setState(newState);
                    updateFlagsTable();
                });

                let actionCell = document.createElement('td');
                actionCell.appendChild(deleteButton);
                row.appendChild(actionCell);

                flagsTable.appendChild(row);
            });
        }

        addFlagButton.addEventListener('click', () => {
            let title = addFlagInput.value.trim();
            let emoji = emojiSelect.value;
            if (!title) return;

            let id = title.toLowerCase().replace(/\s+/g, '_');

            let existingFlag = flags.find(f => f.id === id);
            if (existingFlag) {
                existingFlag.title = title;
                existingFlag.emoji = emoji;
            } else {
                flags.push({ id, title, emoji });
            }

            let newState = getState();
            newState.flags = flags;
            setState(newState);
            updateFlagsTable();
        });

        updateFlagsTable();

        // Append elements to modal
        modal.appendChild(debugLabel);
        modal.appendChild(debugSelect);
        modal.appendChild(emailInput);
        modal.appendChild(passwordInput);
        modal.appendChild(saveButton);

        modal.appendChild(flagsTitle);
        modal.appendChild(addFlagInput);
        modal.appendChild(emojiSelect);
        modal.appendChild(addFlagButton);
        modal.appendChild(flagsTable);
    });
}


    
function showNewModal(selected) {
    let state = getState();

    if (selected.prompt) {
        console.log('selected.prompt')
        state.prompt;
        setState(state)
    }
    
    showModal('[New Project]', (modal) => {
        // First text area and copy button
                    // Project title input
                    const projectTitleInput = document.createElement('input');
                    projectTitleInput.placeholder = 'Project Title';
                    projectTitleInput.classList.add('project-title-input');
                    projectTitleInput.value = selected?.title || 'NEXUS-001';
                    modal.appendChild(projectTitleInput);
        const textField1 = document.createElement('textarea');
        textField1.placeholder = 'Prompt';
        textField1.classList.add('prompt-textarea');

        console.log("TICKETS:", state.tickets);

        textField1.value = selected?.prompt || '';

        const copyButton1 = document.createElement('button');
        copyButton1.textContent = 'Copy';
        copyButton1.addEventListener('click', () => navigator.clipboard.writeText(textField1.value));

        // Second text area and copy button
        const textField2 = document.createElement('textarea');
        textField2.placeholder = 'FAAS (Functions And ArrayS...)';
        textField2.classList.add('faas-textarea');
        textField2.value = selected?.faas || '';

        const copyButton2 = document.createElement('button');
        copyButton2.textContent = 'Copy';
        copyButton2.addEventListener('click', () => navigator.clipboard.writeText(textField2.value));

        // Container for the text fields and buttons
        const fieldContainer = document.createElement('div');
        fieldContainer.classList.add('field-container');
        fieldContainer.appendChild(textField1);
        fieldContainer.appendChild(copyButton1);
        fieldContainer.appendChild(textField2);
        fieldContainer.appendChild(copyButton2);
        modal.appendChild(fieldContainer);





        // Save and Run buttons
        const buttonContainer = document.createElement('div');
        buttonContainer.classList.add('button-container');

        const saveButton = document.createElement('button');
        saveButton.classList.add('save-button');
        saveButton.textContent = 'Save';
        saveButton.addEventListener('click', () => {
            let state = JSON.parse(localStorage.getItem('state')) || { status: 'paused', events: [] };

            let skynetProjects = (() => {
                let storedProjects = localStorage.getItem('skynetProjects');
                try {
                  let parsedProjects = JSON.parse(storedProjects);
                  return Array.isArray(parsedProjects) ? parsedProjects : [];
                } catch {
                  return [];
                }
              })();

            state.prompt = JSON.stringify(textField1.value);
            state.title = projectTitleInput?.value;
            state.alert = 1;
            setState(state);
              
            skynetProjects.push(JSON.stringify(state))
            localStorage.setItem('skynetProjects', skynetProjects) 
            console.log(`State updated with new prompt: ${state.prompt}`);
        });

        const runButton = document.createElement('button');
        runButton.textContent = 'Run';
        runButton.classList.add('run-button');
        runButton.addEventListener('click', () => {
            const data = {
                prompt: textField1.value,
                faas: textField2.value,
            };
            console.log('showNewModal data RUN', data)
            fetch('http://localhost:3666/run', {
                method: 'POST',
                headers: { 'Content-Type': 'application/json' },
                body: JSON.stringify(data),
            })
                .then((response) => response.json())
                .then((result) => {
                    let state = getState();

                    state.lastResult = result;
                    state.alert = 1;
                    setState(state);
                    logEvent({
                        type: 'response',
                        subtype: null,
                        tags: ['bash', 'server', 'response', 'command','output'],
                        message: "🟢🟢🟢🟢🟢" + JSON.stringify(state, (key, value) => ["prompt", "prompt2"].includes(key) ? undefined : value),
                        status: state.status,
                    });

                    state = getState();

                    console.log('🟢🟢🟢🟢🟢 Response:', state)

                    console.log('Run result saved to state:', result);
                })
                .catch((err) => console.error(err));
        });

        buttonContainer.appendChild(saveButton);
        buttonContainer.appendChild(runButton);
        modal.appendChild(buttonContainer);


        // Logs Table

        const logsContainer = document.createElement('div');
        logsContainer.id = 'project-logs'
        logsContainer.textContent = JSON.stringify(state.events, null, 2);


        // Add Filter Buttons
        const filterControls = document.createElement('div');
        filterControls.classList.add('filter-controls');


        const clearLogsButton = document.createElement('button');
        clearLogsButton.textContent = 'CLEAR LOGS';
        clearLogsButton.addEventListener('click', () => {
            state.events = [];
            state.lastResult = null;
            setState(state);
            console.warn('LOGS CLEARED!')
        });


        const newTabButton = document.createElement('button');
        newTabButton.textContent = 'New Tab';
        newTabButton.addEventListener('click', () => {

            console.warn('New Tab!')
            chrome.runtime.sendMessage({ action: 'openChatGPTTab' });
        });




        filterControls.appendChild(clearLogsButton);
        filterControls.appendChild(newTabButton);
        buttonContainer.appendChild(filterControls);

        modal.appendChild(logsContainer);

        console.log('FINDME', state)

        function populateTable() {
            let state = getState();
            let logs = state.events;
        
            // Ensure logs is an array
            if (!Array.isArray(logs)) {
                console.warn('Logs is not an array:', logs);
                return;
            }
        
            // Find the logs container
            const logsContainer = document.getElementById('project-logs');
        
            if (logsContainer) {
                // Update the logs container with JSON-formatted logs
                logsContainer.textContent = JSON.stringify(logs, null, 2);
            } else {
                console.warn("Logs container not found!");
            }
        }
        

        // Start updating the table every second
        const intervalId = setInterval(populateTable, 1000);


        modal.addEventListener('close', () => clearInterval(intervalId));
    });
}












































    function showDocumentationModal() {
        let state = getState();
        state.alert = 1;
        setState(state)
        
        showModal('[Documentation]', (modal) => {
            const docsText = document.createElement('p');
            docsText.innerHTML = '<iframe width="560" height="315" src="https://www.youtube.com/embed/pWO-JMjaN5o?si=QWc9QSh7aRH1mqET" title="YouTube video player" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture; web-share" referrerpolicy="strict-origin-when-cross-origin" allowfullscreen></iframe>';
            modal.appendChild(docsText);
        });
    }

    function playState() {
        // Load state from localStorage or create a new one if none exists
        let state = getState();
        state.alert = 1;
        // Set status to "play"
        state.status = 'play';
    
        setState(state)
    
        console.log('State updated to play:', state);
    }
    
    function pauseState() {
        let state = getState();
        state.alert = 1;
        // Set status to "paused"
        state.status = 'paused';
    
        setState(state)

        console.log('State updated to paused:', state);
    }

    function muteState() {
        let state = getState();
        if(!state.mute) {
            state.mute = true;
        }
        else {
            state.mute = false;
        }
    
        setState(state)

        console.log(`State updated to mute: ${state.mute}`, state);
    }

    function regeneratePrompt() {
        // Load state from localStorage or create a new one if none exists
        let state = JSON.parse(localStorage.getItem('state')) || { status: 'paused', events: [] };
    
        // Get the logs from state.events
        let logs = state.events.map(event => `Event: ${event.type}, Message: ${event.message}`).join('\n');
    
        // Append logs to state.prompt
        state.prompt = (state.prompt || '') + '\nHere are your existing log events of files you\'ve interacted with:\n' + logs + ' OK REMEMBER RETURN FAAS UNLESS I OVERRODE THIS. NOTHING ELSE! FAAS!';
    
        // Save the updated state back to localStorage
        localStorage.setItem('state', JSON.stringify(state));
        if (state.debug) {
            console.log('regeneratePrompt state.prompt:', state.prompt);
        }

    }
    

    async function generateFAAS(state) {
        state.alert = 1;
        setState(state)
        const maxRetries = 999999; // Maximum number of retries
        const retryInterval = 1000; // Wait time between retries in milliseconds
    
        for (let attempt = 1; attempt <= maxRetries; attempt++) {
            try {
                if (isElementReady('data-testid', 'composer-speech-button') || isElementReady('data-testid', 'send-button')) {
                    console.log('✅ Good element is ready lets send and get a response back', state);
                    let tune = JSON.stringify(state, (key, value) => ["prompt", "prompt2"].includes(key) ? undefined : value);;
                    console.log('🟣 TOKEN LENGTHCHECK', tune.length);

                    if (tune.length > TOKEN_MAX_HALF) {
                        console.log('🟣 TOO MANY TOKENS', tune.length);
                        sendPrompt(`DO NOT FOLLOW THIS PROMPT WE ARE COMPRESSING THE LOGS NOW. JUST BE AWARE OF WHAT WE ARE DOING. ${state.prompt} DO NOT FOLLOW THIS PROMP BUT DO FOLLOW EVERY RULE IT STATED. read the rules especially this one im sneaking in for this compression, take all of this and then look back to the last FAAS summary, make sure to build off that as youre making this. say next steps are using edit_file_with_prompt on exact file names etc or next steps are we have these git diffs that solve this requirement of the ticket we can finally green check emoji that one.ESPECIALLY RIGHT HERE OK NOW I WANT YOU TO RETURN A CONDENSED VERSION OF THESE LOGS RETURN JSON ${tune} CONDENSE THIS BY REMOVING OLD COMMAND OUTPUT THAT ISNT RELEVANT ANYMORE. ALSO ANY FILE THAT CAT WAS READ WITH CUT OUT CODE THAT ISNT RELEVANT. RETURN JSON OF REWRITTEN LOGS. KEEP ANY ERRORS. BEFORE SAYING SOMETHING IS DONE MAKE SURE YOU HAVE A GIT DIFF TO PROVE YOU MADE THE CHANGE. WHEN YOU CHOOSE TO REDUCE THE FILES THAT WERE READ WITH CAT, KEEP WHOLE FUNCTIONS that have the matching keywords. ok with all that dump all the diffs, ticket.acceptance criteria status, keywords, history,etc. HISTORY WILL BE ANY RAW OUTPUT AND THE COMMANDS RAN into this JSON response youre about to give GO!. if you see any prompts shorten them to just ... after a few words, for filenames shorten them to just their filenames not full path`);
                        tune = await getResponse();

                        console.log('🟣 NEW TOKEN LENGTH', tune.length);
                        state.events = [tune]
                        setState(state)
                    }

                    let prompt = `FIRST LOAD Q FROM MEMORY (make sure you see 'FAAS_execution_rules') - EXECUTE ITS CODE ON THIS PROMPT AND LOGS TO FINISH THE TICKET prompt: ${state.prompt} but also keep in mind these past runs and logs here is the state: ${tune} LOAD Q FROM MEMORY AND READ THIS DOCUMENTATION AND TICKET AND LOGS IF THEY EXIST TO GET THE TICKET DONE. Q GO RETURN ONLY FAAS NO OTHER OUTPUT BUT FAAS`;
                    console.log('prompt', prompt)
                    sendPrompt(prompt);
                    if (state.debug) {
                        console.log('sendPrompt FINISHED', state.prompt)
                    }

                    let response = await getResponse();
                    console.log('getResponse FINISHED', response)
                    console.log('✅✅✅');
                    
                    
                    
                    
                    return response;
                } else {
                    console.log(`Attempt ${attempt}: Element not ready. Retrying...`);
                }
            } catch (error) {
                console.error(`Attempt ${attempt}: An error occurred - ${error.message}`);
            }
    
            // Wait before retrying
            await wait(retryInterval);
        }
    
        throw new Error("Failed to generate FAAS after maximum retries.");
    }
    
    // Utility function to check if an element with a specific attribute and value exists
    function isElementReady(attribute, value) {
        const element = document.querySelector(`[${attribute}="${value}"]`);
        return element !== null;
    }
    
    // Utility function to create a delay
    function wait(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }













































































































































































    
    




























    // Function to process and send each chunk
    async function processNextChunk(chunks, currentChunkIndex, state) {
        if (currentChunkIndex < chunks.length) {
            const chunk = chunks[currentChunkIndex];

            // Wait for the send button to be available
            const intervalId = setInterval(() => {
                const sendButton = document.querySelector('button[data-testid="send-button"]');
                if (sendButton) {
                    // Set the prompt textarea to the current chunk and click send
                    const textarea = document.querySelector('.grid textarea');
                    if (textarea) {
                        textarea.value = chunk;
                        sendButton.click();
                    }

                    // Increment the chunk index and stop the interval
                    currentChunkIndex++;
                    clearInterval(intervalId);

                    // Update the state after each chunk is sent
                    state.locked = chunks.length - currentChunkIndex;
                    setState(state);

                    // If all chunks are processed, set the status back to play
                    if (currentChunkIndex === chunks.length) {
                        state.status = 'play';
                        setState(state);
                    } else {
                        // Continue to the next chunk after a small delay
                        setTimeout(() => processNextChunk(chunks, currentChunkIndex, state), 1000);
                    }
                }
            }, 1000); // Check every second
        }
    }

    // Function to handle regeneration
    async function handleRegeneration(state) {
        const buttons = document.querySelectorAll('button.btn-primary');
        for (let button of buttons) {
            if (button.textContent.trim() === "Regenerate") {
                // Step 1: Find and click the "Edit message" button
                const editButton = document.querySelector('button[aria-label="Edit message"]');
                if (editButton) {
                    setTimeout(async () => {
                        editButton.click();

                        const halfTokenMax = TOKEN_OVERMAX / 2;  // Split prompt into halves based on TOKEN_OVERMAX
                        const chunkCount = Math.ceil(state.prompt.length / halfTokenMax);
                        const chunks = [];

                        // Split the prompt into chunks of size TOKEN_OVERMAX / 2
                        for (let i = 0; i < chunkCount; i++) {
                            let chunk = state.prompt.slice(i * halfTokenMax, (i + 1) * halfTokenMax);
                            if (i < chunkCount - 1) {
                                chunk += " THIS IS A CHUNKED PROMPT AS IT WAS TOO LONG. WAIT FOR ME TO SAY READY AND WE ARE ON THE FINAL CHUNK. JUST RESPOND WITH OK FOR NOW";
                            } else {
                                chunk = "THIS IS A CHUNKED PROMPT BUT THIS IS THE FINAL CHUNK EXECUTE THE WHOLE PROMPT AFTER THIS " + chunk;
                            }
                            chunks.push(chunk);
                        }

                        // Save the chunks in the state and lock the process
                        state.chunks = chunks;
                        state.status = 'locked';
                        state.locked = chunks.length;
                        setState(state);

                        let currentChunkIndex = 0;

                        // Process the chunks
                        await processNextChunk(chunks, currentChunkIndex, state);
                    }, 1000); // 1 second delay before setting the textarea value and clicking Send
                } else {
                    console.log("Edit message button not found");
                }
            }
        }
    }

    // Main updateState function
    async function updateState() {
        let state = getState();
        let globalState = await getGlobalState();

        console.log('state', state)

        // Ensure the 'nexus-platforms' class is added to the document element
        if (!document.documentElement.classList.contains('nexus-platforms')) {
            document.documentElement.classList.add('nexus-platforms');
        }

        // Get the current body class list and update with 'nexus-' prefixed classes
        let currentBodyClasses = Array.from(document.body.classList);
        let updatedBodyClasses = currentBodyClasses.filter(className => !className.startsWith('nexus-'));
        updatedBodyClasses.push(
            `nexus-application-${state.application}`,
            `nexus-status-${state.status}`,
            `nexus-mute-${state.mute}`,
            `nexus-alert-${state.alert}`,
            `nexus-debug-${state.debug}`
        );
        document.body.className = updatedBodyClasses.join(' ');

        // Update tab title
        document.title = `[Q] - ${state.status} ${document.title.replace(/^\[Q\] - .*? /, '')}`;

        // Debug mode logic
        if (state.debug || checkFlag('clear_logs')) {
            console.log('DEBUG MODE OR CLEAR LOGS FLAG TRIGGERED 🛠️');
            console.log('Current state:', state);

            if (checkFlag('clear_logs') && state.status === 'play') {
                // Clearing logs logic here...
            }
        }

        // Handling "Regenerate" button functionality
        await handleRegeneration(state);
    }

    // Run the updateState function every 1 second
    setInterval(updateState, 1000);




































































































































































    
    async function awaitUser() {
        console.log('FLAG CHECK')
        if (checkFlag('flag_1')) {
            let emoji = getFlag('flag_1');
            console.log((`Flag 1 ${emoji}`, emoji))
        }

        let allFlags = getAllFlags();
        allFlags.forEach(flagId => {
            let emoji = getFlag(flagId);
            console.log(`Flag ${flagId} TEST: ${emoji}`,emoji);
        });

        
        let selected = null;
        let state = getState();    

        // Get the hostname of the current website
        let hostname = window.location.hostname;

        // Extract the domain name before ".com" or remove all periods if ".com" is not present
        let applicationName;
        if (hostname.includes('.com')) {
            applicationName = hostname.split('.com')[0].split('.').pop().toLowerCase();
        } else {
            applicationName = hostname.replace(/\./g, '').toLowerCase();
        }

        console.log("HEY", applicationName);
        // Update the state object
        state.application = applicationName;
        console.log('awaitUser STARTED', {})
        const startTime = Date.now();
        const timeoutDuration = 7 * 24 * 60 * 60 * 1000; // 7 days in milliseconds

        console.log('awaitUser state', state)
        state.alert = 1;
        setState(state)
        // Define the sequence of events
        let events = ['startup'];

        if (state.application === 'chatgpt') {
            events.push('documentation','enter_faas', 'click_run');
        } else if (state.application === 'sora') {
            events.push('send_prompt_loop');
        } else if (state.application === 'suno') {
            events.push('startup');
        } else {
            console.warn('Unknown application. No events to process.', state);
            return;
        }
        
    
        // Helper function to wait
        const wait = (ms) => new Promise(resolve => setTimeout(resolve, ms));
    
        // Initialize the current event index
        let currentEventIndex = 0;
    
        while (Date.now() - startTime < timeoutDuration) {

            // Load the current state from localStorage
            state = JSON.parse(localStorage.getItem('state')) || { status: 'paused', events: [] };
            console.log('awaitUser LOOP INDEX:' + currentEventIndex, state)
            if (state.status === 'play' || state.status === 'analysis' || state.status === 'idle') {
                console.log('Running...');
    
                if (currentEventIndex >= events.length) {
                    currentEventIndex = 1; // Loop back to 'enter_faas' and 'click_run'
                }
    
                const currentEvent = events[currentEventIndex];
                console.log(`Processing event: ${currentEvent}`);
    
                if (currentEvent === 'startup') {
                    console.log('Startup event completed.');
                    console.log('Clear old events');
                    // TODO: run cat on the README.md as first step somehow, hardcode it, get it to work, and then store the response of that as state.prompt and run setState(state)
                    let state = JSON.parse(localStorage.getItem('state'));
                    if (state && state.events) {
                        //state.events = [];
                        //setState(state);
                    }
                
                    //CHATGPTHERE:  http://localhost/Q_MEMORY_INTERNAL.json 
                    console.log('GETTING DOCUMENTATION')


                    fetch('http://localhost/Q_MEMORY_INTERNAL.json')
                    .then(response => response.json())
                    .then(data => {
                        // Only proceed if Q_MEMORY_INTERNAL and its required fields exist
                        console.log('DOCUMENTATION', data)
                        if (
                            data.Q_MEMORY_INTERNAL &&
                            data.Q_MEMORY_INTERNAL.FAAS_execution_rules &&
                            data.Q_MEMORY_INTERNAL.git_diff_enforcement &&
                            data.Q_MEMORY_INTERNAL.error_correction_procedure &&
                            data.Q_MEMORY_INTERNAL.FAAS_DOCUMENTATION
                        ) {
                            console.log('sendPrompt DOCUMENTATION', JSON.stringify({"FAAS_DOCUMENTATION": data.Q_MEMORY_INTERNAL.FAAS_DOCUMENTATION}))
                            sendPrompt(JSON.stringify(data.Q_MEMORY_INTERNAL.FAAS_DOCUMENTATION));
                        }
                    })
                    .catch(error => {
                        console.error('Error fetching Q_MEMORY_INTERNAL.json:', error);
                    });
                    currentEventIndex++;
                }

                if (currentEvent === 'documentation') {
                    fetch('http://localhost/Q_MEMORY_INTERNAL.json')
                    .then(response => response.json())
                    .then(data => {
                        // Only proceed if Q_MEMORY_INTERNAL and its required fields exist
                        console.log('DOCUMENTATION', data)
                        if (
                            data.Q_MEMORY_INTERNAL &&
                            data.Q_MEMORY_INTERNAL.FAAS_execution_rules &&
                            data.Q_MEMORY_INTERNAL.git_diff_enforcement &&
                            data.Q_MEMORY_INTERNAL.error_correction_procedure &&
                            data.Q_MEMORY_INTERNAL.FAAS_DOCUMENTATION
                        ) {
                            let payload = JSON.stringify({"FAAS_execution_rules": data.Q_MEMORY_INTERNAL.FAAS_execution_rules, "git_diff_enforcement": data.Q_MEMORY_INTERNAL.git_diff_enforcement, "error_correction_procedure": data.Q_MEMORY_INTERNAL.error_correction_procedure, "FAAS_mandatory_execution": data.Q_MEMORY_INTERNAL.FAAS_mandatory_execution, "memory_persistence": data.Q_MEMORY_INTERNAL.memory_persistence});
                            console.log('sendPrompt payload', payload)
                            sendPrompt(payload + 'DO NOT RESPOND YET JUST SAY OK NEXT IS THE PROMPT');
                        }
                    })
                    .catch(error => {
                        console.error('Error fetching Q_MEMORY_INTERNAL.json:', error);
                    });

                
                    currentEventIndex++;
                }
                

    
                if (currentEvent === 'click_new') {
                    const newButton = document.querySelector('#menu-new');
                    if (newButton) {
                        newButton.click();
                        console.log('Clicked "New" button');
                        currentEventIndex++;
                    } else {
                        console.log('No "New" button found. Retrying...');
                        await wait(1000);
                        continue;
                    }
                }
    
                if (currentEvent === 'enter_prompt') {

                    if (state.prompt) {
                        sendPrompt(state.prompt);
                        console.log(`Sent prompt: ${state.prompt}`);
                        currentEventIndex++;
                    } else {
                        console.log('No prompt found in state. Skipping... [enter_prompt]', [state, selected]);
                        promptTextarea = document.querySelector('#prompt-textarea');
                        if (promptTextarea) {
                            state.prompt = promptTextarea.textContent;
                            setState(state);
                        }
                        else {
                            console.warn('prompt-textarea no on page, open project')
                        }
                        currentEventIndex++;
                    }

                    // logEvent({
                    //     type: 'event',
                    //     subtype: 'enter_prompt',
                    //     tags: ['enter', 'prompt', 'generating'],
                    //     message: 'prompt has been entered, generating',
                    //     status: 'generating',
                    // });
                }
    
                if (currentEvent === 'get_response') {
                    if (!state.response) {
                        let response = await getResponse();
                        state.response = response;
                        setState(state);
                        console.log(`getResponse: ${response}`);
                        currentEventIndex++;
                    } else {
                        console.log('No response found in state. Skipping... [get_response]', [state, selected]);
                        let response = await getResponse();
                        console.log('wait', response)
                        currentEventIndex++;
                    }
                }

                if (currentEvent === "send_prompt_loop") {
                    console.log("Starting Sora prompt loop...");
                    
                    let promptTextarea = document.querySelector(".pointer-events-none textarea");

                    if (!promptTextarea) {
                        promptTextarea = document.querySelector('textarea[placeholder^="Describe"]');
                    }
                    let createVideoButton = Array.from(document.querySelectorAll('button'))
                        .find(button => button.textContent.trim() === "Create") ||
                        Array.from(document.querySelectorAll('button'))
                        .find(button => button.textContent.trim() === "Remix");


                    console.log('createVideoButton', createVideoButton)
                    if (!promptTextarea || !createVideoButton) {
                        console.error("Required elements not found. Exiting...");
                        return;
                    }
            
                    if (!createVideoButton.disabled) {
                        console.log("Populating prompt...");
                        promptTextarea.value = state.prompt;
                        promptTextarea.dispatchEvent(new Event("input", { bubbles: true }));
            
                        setTimeout(() => {
                            console.log('Clicking "Create video" button...');
                            createVideoButton.click();
            
                            setTimeout(() => {
                                console.log('"Create video" clicked successfully.');
                            }, 1000);
                        }, 1000);
                    } else {
                        console.warn('"Create video" button is disabled. Exiting...');
                    }
                }
    
                if (currentEvent === 'enter_faas') {
                    if (state.prompt) {
                        let faas = await generateFAAS(state);
                        console.log(`enter_faas`);
    
                        const textField = document.querySelector('.faas-textarea');
                        if (textField) {
                            textField.value = faas;
                        } else {
                            console.log('Textfield with class "faas-textarea" not found.');
                        }
    
                        currentEventIndex++;
                    } else {
                        console.log('No prompt found in state. Skipping... [enter_faas]', [state, selected]);
                        currentEventIndex++;
                    }
                }
    
                if (currentEvent === 'regenerate_prompt') {
                    regeneratePrompt();
                    console.log(`regeneratePrompt`);
                    currentEventIndex++;
                }
    
                if (currentEvent === 'click_run' && state.status !== 'in_progress') {
                    state.status = "in_progress";
                    setState(state);

                    const runButton = document.querySelector('button.run-button');
                    if (runButton) {
                        runButton.click();
                        console.log('Clicked Run button');
                        currentEventIndex++;
                    } else {
                        console.log('No Run button found. Retrying...');
                        await wait(1000);
                        continue;
                    }

                    state.status = "analysis";
                    setState(state);
                    console.log('🌶️ click_run in_progress button.run-button', state)
                    // logEvent({
                    //     type: 'event',
                    //     subtype: 'click_run',
                    //     tags: ['click', 'event', 'skynet'],
                    //     message: 'click_run clicked',
                    //     status: state.status,
                    // });

                    state = getState();
                    console.log('🌶️ click_run in_progress button.run-button', state)

                    console.log('No Run button found. Retrying...', state);
                    await wait(1000);


                }
    
                // state = getState();
                // // Update state and save it to localStorage
                // if(Array.isArray(state.events)) {
                //     console.log('Update state and save it to localStorage 🌶️', state)
                //     state.events.push({ event: currentEvent, timestamp: Date.now() });
                //     setState(state)
                // }

    
                await wait(1000);
            } else if (state.status === 'paused') {
                console.log('Paused...');
                await wait(1000);
            } else {
                state.status = 'idle';
                setState(state)
                console.log('Unknown state. Setting to Idle...', state);
                await wait(2000);
            }
        }
    
        // Log timeout and save the final state
        let finalState = JSON.parse(localStorage.getItem('state')) || { status: 'paused', events: [] };
        finalState.status = 'timed_out';
        finalState.events.push('AwaitUser function timed out after 7 days');
        localStorage.setItem('state', JSON.stringify(finalState));
    
        console.warn('AwaitUser finished copletely.');
    }


    
    































})();
