import json
import os
import sys
from openai import OpenAI

# Set your API key
API_KEY = "sk-proj-4FAxkz41b0eIJMXLPpLST3BlbkFJySTy2OLM9FFSC81YVxmG"

client = OpenAI(api_key=API_KEY)

def extract_json_from_response(raw_response):
    """Extracts and cleans JSON from OpenAI response."""
    raw_response = raw_response.strip()

    # Remove surrounding code block formatting
    if "```" in raw_response:
        raw_response = raw_response.split("```", 1)[1]  # Remove first ```
        raw_response = raw_response.split("```", 1)[0]  # Remove text after second ```

    # Trim everything before first { or [
    json_start = min(raw_response.find("{") if "{" in raw_response else float("inf"),
                     raw_response.find("[") if "[" in raw_response else float("inf"))
    if json_start != float("inf"):
        raw_response = raw_response[json_start:]

    # Trim everything after last } or ]
    json_end = max(raw_response.rfind("}"), raw_response.rfind("]"))
    if json_end != -1:
        raw_response = raw_response[:json_end + 1]

    return raw_response.strip()

def get_patch_from_openai(file_path, file_content, prompt):
    """Generates a JSON patch file from OpenAI using full-string find & replace (no line numbers)."""

    example_patch = {
        "file_path": file_path,
        "patch": [
            {
                "before": "    let existingContainer = document.getElementById('skynet-container');",
                "after": "    let existingContainer = document.querySelector('#skynet-container');"
            },
            {
                "before": "    console.log('Initializing...');",
                "after": "    console.log('Skynet is taking over...');"
            }
        ]
    }

    system_message = (
        "You are an AI that modifies files and returns structured JSON patches for full-string replacements.\n\n"
        "Do **not** return line numbers. Instead, return a **list of replacements**, each containing:\n"
        "- `before`: The full original line (including spaces and indentation).\n"
        "- `after`: The modified version.\n\n"
        "**IMPORTANT RULES:**\n"
        "1. The `before` string must match **exactly** in the file.\n"
        "2. If multiple occurrences of the same `before` string exist, throw a warning (do not automatically replace all).\n"
        "3. Ensure that replacements do not alter unintended parts of the code.\n\n"
        "Example JSON Patch:\n"
        f"{json.dumps(example_patch, indent=4)}"
    )

    user_message = {
        "instruction": prompt,
        "file_path": file_path,
        "original_content": file_content
    }

    response = client.chat.completions.create(
        model="gpt-4-turbo",
        messages=[
            {"role": "system", "content": system_message},
            {"role": "user", "content": json.dumps(user_message)}
        ],
        max_tokens=4000
    )

    raw_response = response.choices[0].message.content.strip()
    clean_json = extract_json_from_response(raw_response)

    # Debugging: Print cleaned response
    print("\n🔹 [DEBUG] Cleaned JSON Response:\n", clean_json)

    # Parse JSON safely
    try:
        patch_data = json.loads(clean_json)
        if not isinstance(patch_data, dict) or "patch" not in patch_data:
            print("❌ [ERROR] OpenAI response does not contain a valid 'patch'.")
            return None
        return patch_data
    except json.JSONDecodeError:
        print("❌ [ERROR] OpenAI response is not valid JSON.")
        return None

def save_patch(file_path, patch_data):
    """Saves the JSON patch file to disk."""
    patch_file_path = f"{file_path}.patch.json"
    
    with open(patch_file_path, "w", encoding="utf-8") as patch_file:
        json.dump(patch_data, patch_file, indent=4)

    print(f"✅ Patch saved to {patch_file_path}")

def apply_patch(file_path, patch_data):
    """Applies a JSON patch using full-string replacements (find & replace)."""
    try:
        with open(file_path, "r", encoding="utf-8") as file:
            content = file.read()
    except FileNotFoundError:
        print(f"❌ [ERROR] File '{file_path}' not found.")
        return

    modified_content = content
    for change in patch_data["patch"]:
        before = change["before"]
        after = change["after"]

        count = content.count(before)

        if count == 0:
            print(f"⚠️ [WARNING] No match found for: {before}")
        elif count > 1:
            print(f"⚠️ [WARNING] Found {count} matches for: {before}. Only replacing the first occurrence.")
        
        modified_content = modified_content.replace(before, after, 1)

    with open(file_path, "w", encoding="utf-8") as file:
        file.write(modified_content)

    print(f"✅ Patch applied successfully to {file_path}")

def edit_file_with_prompt(file_path, prompt):
    """Generates and applies a JSON patch to a file based on a prompt using find & replace."""
    try:
        with open(file_path, "r", encoding="utf-8") as file:
            file_content = file.read()
    except FileNotFoundError:
        print(f"❌ [ERROR] File '{file_path}' not found.")
        return

    # Generate patch
    patch_data = get_patch_from_openai(file_path, file_content, prompt)
    if not patch_data:
        return

    # Save patch file
    # save_patch(file_path, patch_data)

    # Apply the patch to the file
    apply_patch(file_path, patch_data)

if __name__ == "__main__":
    if len(sys.argv) != 3:
        print("Usage: python3 edit_file_with_prompt.py <file_path> <prompt>")
        sys.exit(1)

    edit_file_with_prompt(sys.argv[1], sys.argv[2])
